"""
This module exports routines of PHCpack to manipulate
positive dimensional solution sets of polynomial systems.
The embed functions add slack variables and random hyperplanes.
The number of slack variables equals the number of random hyperplanes,
which in turn equals the dimension of the solution set.
The drop functions remove the added slack variables from the polynomials
and the coordinates of the solutions.
Given a witness set and a point, a homotopy membership determines whether
the point belongs to the solution set represented by the witness set.
"""

def standard_embed(nvar, topdim, pols, verbose_level=0):
    r"""
    Given in *pols* a list of strings representing polynomials in *nvar*
    variables, with coefficients in standard double precision,
    this function returns an embedding of *pols* of dimension *topdim*.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    from phcpy.phcpy2c3 import py2c_embed_standard_system
    from phcpy.interface import store_standard_system, load_standard_system
    store_standard_system(pols, nbvar=nvar)
    py2c_embed_standard_system(topdim,verbose_level)
    return load_standard_system()

def dobldobl_embed(nvar, topdim, pols, verbose_level=0):
    r"""
    Given in *pols* a list of strings that represent polynomials in *nvar*
    variables, with coefficients in double double precision,
    this function returns an embedding of *pols* of dimension *topdim*.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    from phcpy.phcpy2c3 import py2c_embed_dobldobl_system
    from phcpy.interface import store_dobldobl_system, load_dobldobl_system
    store_dobldobl_system(pols, nbvar=nvar)
    py2c_embed_dobldobl_system(topdim,verbose_level)
    return load_dobldobl_system()

def quaddobl_embed(nvar, topdim, pols, verbose_level=0):
    r"""
    Given in *pols* a list of strings that represent polynomials in *nvar*
    variables, with coefficients in quad double precision,
    this function returns an embedding of *pols* of dimension topdim.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    from phcpy.phcpy2c3 import py2c_embed_quaddobl_system
    from phcpy.interface import store_quaddobl_system, load_quaddobl_system
    store_quaddobl_system(pols, nbvar=nvar)
    py2c_embed_quaddobl_system(topdim,verbose_level)
    return load_quaddobl_system()

def standard_laurent_embed(nvar, topdim, pols, verbose_level=0):
    r"""
    Given in *pols* a list of strings representing Laurent polynomials 
    in *nvar* variables, with coefficients in standard double precision,
    this function returns an embedding of *pols* of dimension *topdim*.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    from phcpy.phcpy2c3 import py2c_embed_standard_Laurent_system
    from phcpy.interface import store_standard_laurent_system
    from phcpy.interface import load_standard_laurent_system
    store_standard_laurent_system(pols, nbvar=nvar)
    py2c_embed_standard_Laurent_system(topdim,verbose_level)
    return load_standard_laurent_system()

def dobldobl_laurent_embed(nvar, topdim, pols, verbose_level=0):
    r"""
    Given in *pols* a list of strings that represent Laurent polynomials 
    in *nvar* variables, with coefficients in double double precision,
    this function returns an embedding of *pols* of dimension *topdim*.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    from phcpy.phcpy2c3 import py2c_embed_dobldobl_Laurent_system
    from phcpy.interface import store_dobldobl_laurent_system
    from phcpy.interface import load_dobldobl_laurent_system
    store_dobldobl_laurent_system(pols, nbvar=nvar)
    py2c_embed_dobldobl_Laurent_system(topdim,verbose_level)
    return load_dobldobl_laurent_system()

def quaddobl_laurent_embed(nvar, topdim, pols, verbose_level=0):
    r"""
    Given in *pols* a list of strings that represent Laurent polynomials 
    in *nvar* variables, with coefficients in quad double precision,
    this function returns an embedding of *pols* of dimension topdim.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    from phcpy.phcpy2c3 import py2c_embed_quaddobl_Laurent_system
    from phcpy.interface import store_quaddobl_laurent_system
    from phcpy.interface import load_quaddobl_laurent_system
    store_quaddobl_laurent_system(pols, nbvar=nvar)
    py2c_embed_quaddobl_Laurent_system(topdim,verbose_level)
    return load_quaddobl_laurent_system()

def embed(nvar, topdim, pols, precision='d', verbose_level=0):
    r"""
    Given in *pols* a list of strings that represent polynomials in *nvar*
    variables, this function returns an embedding of *pols* 
    of dimension *topdim*.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    The default *precision* of the coefficients is 'd', for standard double
    precision.  For double double and quad double precision, set the value
    of *precision* to 'dd' or 'qd' respectively.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the embed procedure will be listed.
    """
    if(precision == 'd'):
        return standard_embed(nvar, topdim, pols, verbose_level)
    elif(precision == 'dd'):
        return dobldobl_embed(nvar, topdim, pols, verbose_level)
    elif(precision == 'qd'):
        return quaddobl_embed(nvar, topdim, pols, verbose_level)
    else:
        print('wrong argument for precision')
        return None

def laurent_embed(nvar, topdim, pols, precision='d', verbose_level=0):
    r"""
    Given in *pols* a list of strings that represent Laurent polynomials
    in *nvar* variables, this function returns an embedding of *pols* 
    of dimension *topdim*.
    The *topdim* is the top dimension which equals the expected highest
    dimension of a component of the solution set of the system of polynomials.
    The default *precision* of the coefficients is 'd', for standard double
    precision.  For double double and quad double precision, set the value
    of *precision* to 'dd' or 'qd' respectively.
    If *verbose_level* is larger than 0, then the names of the procedures
    called in the running of the embed procedure will be listed.
    """
    if(precision == 'd'):
        return standard_laurent_embed(nvar, topdim, pols, verbose_level)
    elif(precision == 'dd'):
        return dobldobl_laurent_embed(nvar, topdim, pols, verbose_level)
    elif(precision == 'qd'):
        return quaddobl_laurent_embed(nvar, topdim, pols, verbose_level)
    else:
        print('wrong argument for precision')
        return None

def witness_set_of_hypersurface(nvar, hpol, precision='d'):
    r"""
    Given in *hpol* the string representation of a polynomial
    in *nvar* variables (ending with a semicolon),
    on return is an embedded system and its solutions
    which represents a witness set for *hpol*.
    The number of solutions on return should equal
    the degree of the polynomial in *hpol*.
    Three different precisions are supported, by default double ('d'),
    or otherwise double double ('dd') or quad double ('qd').
    """
    if(precision == 'd'):
        from phcpy.phcpy2c3 import py2c_standard_witset_of_hypersurface
        from phcpy.interface import load_standard_system
        from phcpy.interface import load_standard_solutions
        py2c_standard_witset_of_hypersurface(nvar, len(hpol), hpol)
        return (load_standard_system(), load_standard_solutions())
    elif(precision == 'dd'):
        from phcpy.phcpy2c3 import py2c_dobldobl_witset_of_hypersurface
        from phcpy.interface import load_dobldobl_system
        from phcpy.interface import load_dobldobl_solutions
        py2c_dobldobl_witset_of_hypersurface(nvar, len(hpol), hpol)
        return (load_dobldobl_system(), load_dobldobl_solutions())
    elif(precision == 'qd'):
        from phcpy.phcpy2c3 import py2c_quaddobl_witset_of_hypersurface
        from phcpy.interface import load_quaddobl_system
        from phcpy.interface import load_quaddobl_solutions
        py2c_quaddobl_witset_of_hypersurface(nvar, len(hpol), hpol)
        return (load_quaddobl_system(), load_quaddobl_solutions())
    else:
        print('wrong argument for precision')
        return None

def witness_set_of_laurent_hypersurface(nvar, hpol, precision='d'):
    r"""
    Given in *hpol* the string representation of a laurent polynomial
    in *nvar* variables (ending with a semicolon),
    on return is an embedded system and its solutions
    which represents a witness set for *hpol*.
    The number of solutions on return may differ from the actual degree
    of hpol if the polynomial represented by hpol has negative exponents.
    Three different precisions are supported, by default double ('d'),
    or otherwise double double ('dd') or quad double ('qd').
    """
    if(precision == 'd'):
        from phcpy.phcpy2c3 import py2c_standard_witset_of_Laurent_hypersurface
        from phcpy.interface import load_standard_laurent_system
        from phcpy.interface import load_standard_solutions
        py2c_standard_witset_of_Laurent_hypersurface(nvar, len(hpol), hpol)
        return (load_standard_laurent_system(), load_standard_solutions())
    elif(precision == 'dd'):
        from phcpy.phcpy2c3 import py2c_dobldobl_witset_of_Laurent_hypersurface
        from phcpy.interface import load_dobldobl_laurent_system
        from phcpy.interface import load_dobldobl_solutions
        py2c_dobldobl_witset_of_Laurent_hypersurface(nvar, len(hpol), hpol)
        return (load_dobldobl_laurent_system(), load_dobldobl_solutions())
    elif(precision == 'qd'):
        from phcpy.phcpy2c3 import py2c_quaddobl_witset_of_Laurent_hypersurface
        from phcpy.interface import load_quaddobl_laurent_system
        from phcpy.interface import load_quaddobl_solutions
        py2c_quaddobl_witset_of_Laurent_hypersurface(nvar, len(hpol), hpol)
        return (load_quaddobl_laurent_system(), load_quaddobl_solutions())
    else:
        print('wrong argument for precision')

def drop_variable_from_standard_polynomials(pols, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *pols* of strings that represent polynomials
    in several variables, with coefficients in standard double precision.
    Note that the system in *pols* must be square.
    """
    from phcpy.phcpy2c3 import py2c_syscon_standard_drop_variable_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_standard_system, load_standard_system
    store_standard_system(pols)
    py2c_syscon_standard_drop_variable_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_standard_system()

def drop_variable_from_dobldobl_polynomials(pols, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *pols* of strings that represent polynomials
    in several variables, with coefficients in double double precision.
    Note that the system in *pols* must be square.
    """
    from phcpy.phcpy2c3 import py2c_syscon_dobldobl_drop_variable_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_dobldobl_system, load_dobldobl_system
    store_dobldobl_system(pols)
    py2c_syscon_dobldobl_drop_variable_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_dobldobl_system()

def drop_variable_from_quaddobl_polynomials(pols, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *pols* of strings that represent polynomials
    in several variables, with coefficients in quad double precision.
    Note that the system in *pols* must be square.
    """
    from phcpy.phcpy2c3 import py2c_syscon_quaddobl_drop_variable_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_quaddobl_system, load_quaddobl_system
    store_quaddobl_system(pols)
    py2c_syscon_quaddobl_drop_variable_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_quaddobl_system()

def drop_variable_from_standard_laurent_polynomials(pols, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *pols* of strings that represent Laurent polynomials
    in several variables, with coefficients in standard double precision.
    Note that the system in *pols* must be square.
    """
    from phcpy.phcpy2c3 \
    import py2c_syscon_standard_Laurent_drop_variable_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_standard_laurent_system
    from phcpy.interface import load_standard_laurent_system
    store_standard_laurent_system(pols)
    py2c_syscon_standard_Laurent_drop_variable_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_standard_laurent_system()

def drop_variable_from_dobldobl_laurent_polynomials(pols, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *pols* of strings that represent Laurent polynomials
    in several variables, with coefficients in double double precision.
    Note that the system in *pols* must be square.
    """
    from phcpy.phcpy2c3 \
    import py2c_syscon_dobldobl_Laurent_drop_variable_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_dobldobl_laurent_system
    from phcpy.interface import load_dobldobl_laurent_system
    store_dobldobl_laurent_system(pols)
    py2c_syscon_dobldobl_Laurent_drop_variable_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_dobldobl_laurent_system()

def drop_variable_from_quaddobl_laurent_polynomials(pols, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *pols* of strings that represent Laurent polynomials
    in several variables, with coefficients in quad double precision.
    Note that the system in *pols* must be square.
    """
    from phcpy.phcpy2c3 \
    import py2c_syscon_quaddobl_Laurent_drop_variable_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_quaddobl_laurent_system
    from phcpy.interface import load_quaddobl_laurent_system
    store_quaddobl_laurent_system(pols)
    py2c_syscon_quaddobl_Laurent_drop_variable_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_quaddobl_laurent_system()

def drop_coordinate_from_standard_solutions(sols, nbvar, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *sols* of strings that represent solutions
    in *nbvar* variables, in standard double precision.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_solcon_standard_drop_coordinate_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    py2c_syscon_clear_symbol_table()
    store_standard_solutions(nbvar, sols)
    py2c_solcon_standard_drop_coordinate_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_standard_solutions()

def drop_coordinate_from_dobldobl_solutions(sols, nbvar, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *sols* of strings that represent solutions
    in *nbvar* variables, in double double precision.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_solcon_dobldobl_drop_coordinate_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    py2c_syscon_clear_symbol_table()
    store_dobldobl_solutions(nbvar, sols)
    py2c_solcon_dobldobl_drop_coordinate_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_dobldobl_solutions()

def drop_coordinate_from_quaddobl_solutions(sols, nbvar, svar):
    r"""
    Removes the variable with symbol in the string *svar*
    from the list *sols* of strings that represent solutions
    in *nbvar* variables, in quad double precision.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_solcon_quaddobl_drop_coordinate_by_name
    from phcpy.phcpy2c3 import py2c_syscon_remove_symbol_name
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    py2c_syscon_clear_symbol_table()
    store_quaddobl_solutions(nbvar, sols)
    py2c_solcon_quaddobl_drop_coordinate_by_name(len(svar), svar)
    py2c_syscon_remove_symbol_name(len(svar), svar)
    return load_quaddobl_solutions()

def standard_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list *point*,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in standard double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    """
    from phcpy.interface import store_standard_witness_set
    from phcpy.phcpy2c3 import py2c_witset_standard_membertest as membtest
    store_standard_witness_set(len(wsys), dim, wsys, gpts)
    nvr = len(point)//2
    spt = str(point)
    nbc = len(spt)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, spt)
    return (result[2] == 1)

def dobldobl_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list *point*,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in double double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    """
    from phcpy.interface import store_dobldobl_witness_set
    from phcpy.phcpy2c3 import py2c_witset_dobldobl_membertest as membtest
    store_dobldobl_witness_set(len(wsys), dim, wsys, gpts)
    nvr = len(point)//4
    spt = str(point)
    nbc = len(spt)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, spt)
    return (result[2] == 1)

def quaddobl_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list point,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in quad double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    """
    from phcpy.interface import store_quaddobl_witness_set
    from phcpy.phcpy2c3 import py2c_witset_quaddobl_membertest as membtest
    store_quaddobl_witness_set(len(wsys), dim, wsys, gpts)
    nvr = len(point)//8
    spt = str(point)
    nbc = len(spt)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, spt)
    return (result[2] == 1)

def standard_laurent_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding Laurent
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list *point*,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in standard double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    """
    from phcpy.interface import store_standard_laurent_witness_set
    from phcpy.phcpy2c3 \
    import py2c_witset_standard_Laurent_membertest as membtest
    store_standard_laurent_witness_set(len(wsys), dim, wsys, gpts)
    nvr = len(point)//2
    spt = str(point)
    nbc = len(spt)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, spt)
    return (result[2] == 1)

def dobldobl_laurent_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding Laurent
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list *point*,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in double double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    """
    from phcpy.interface import store_dobldobl_laurent_witness_set
    from phcpy.phcpy2c3 \
    import py2c_witset_dobldobl_Laurent_membertest as membtest
    store_dobldobl_laurent_witness_set(len(wsys), dim, wsys, gpts)
    nvr = len(point)//4
    spt = str(point)
    nbc = len(spt)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, spt)
    return (result[2] == 1)

def quaddobl_laurent_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding Laurent
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list point,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in quad double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    """
    from phcpy.interface import store_quaddobl_laurent_witness_set
    from phcpy.phcpy2c3 \
    import py2c_witset_quaddobl_Laurent_membertest as membtest
    store_quaddobl_laurent_witness_set(len(wsys), dim, wsys, gpts)
    nvr = len(point)//8
    spt = str(point)
    nbc = len(spt)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, spt)
    return (result[2] == 1)

def membertest(wsys, gpts, dim, point, evatol=1.0e-6, memtol=1.0e-6, \
    verbose=True, precision='d', tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list *point*,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    The default working *precision* is double 'd'.  Other levels of precision
    are double double precision 'dd' and quad double precision 'qd'.
    There are two tolerances: *evatol* is the tolerance on the residual
    of the evaluation of the polynomial equations at the test point.
    If the residual of the evaluation is not less than *evatol*,
    then the membertest returns False.  Otherwise, the homotopy
    membership test is called and the *memtol* is used to compare
    the coordinates of the point with the newly computed generic points.
    If there is a match between the coordinates within the given
    tolerance *memtol*, then True is returned.
    """
    if(precision == 'd'):
        return standard_membertest(wsys, gpts, dim, point, \
                                   evatol, memtol, verbose, tasks)
    elif(precision == 'dd'):
        return dobldobl_membertest(wsys, gpts, dim, point, \
                                   evatol, memtol, verbose, tasks)
    elif(precision == 'qd'):
        return quaddobl_membertest(wsys, gpts, dim, point, \
                                   evatol, memtol, verbose, tasks)
    else:
        print('wrong argument for precision')
        return None

def laurent_membertest(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, precision='d', tasks=0):
    r"""
    Applies the homotopy membership test for a point to belong to
    a witness set of dimension *dim*, given by an embedding Laurent
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the list *point*,
    as a list of doubles, with the real and imaginary part of each
    coordinate of the point.  By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    The default working *precision* is double 'd'.  Other levels of precision
    are double double precision 'dd' and quad double precision 'qd'.
    There are two tolerances: *evatol* is the tolerance on the residual
    of the evaluation of the Laurent system at the test point.
    If the residual of the evaluation is not less than *evatol*,
    then the membertest returns False.  Otherwise, the homotopy
    membership test is called and the *memtol* is used to compare
    the coordinates of the point with the newly computed generic points.
    If there is a match between the coordinates within the given
    tolerance *memtol*, then True is returned.
    """
    if(precision == 'd'):
        return standard_laurent_membertest\
                   (wsys, gpts, dim, point, evatol, memtol, verbose, tasks)
    elif(precision == 'dd'):
        return dobldobl_membertest\
                   (wsys, gpts, dim, point, evatol, memtol, verbose, tasks)
    elif(precision == 'qd'):
        return quaddobl_membertest\
                   (wsys, gpts, dim, point, evatol, memtol, verbose, tasks)
    else:
        print('wrong argument for precision')
        return None

def standard_ismember(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a *point* to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the string *point*,
    which is the string representation of a solution in PHCpack format,
    with symbols of the variables before the values of the coordinates.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in standard double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    Returns a tuple of two booleans.  The first boolean is True if the
    point satisfies the equations, otherwise it is False.  The second
    boolean is True if the point belongs to the witness set,
    otherwise, the second boolean is False.
    """
    from phcpy.interface import store_standard_witness_set
    from phcpy.phcpy2c3 import py2c_witset_standard_ismember as membtest
    store_standard_witness_set(len(wsys), dim, wsys, gpts)
    nbc = len(point)
    nvr = len(wsys) - dim # test point should not have slack variables
    if verbose:
        print('calling standard_ismember with test point :')
        print(point)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, point)
    (fail, onpolsys, inwitset) = result
    return (onpolsys, inwitset)

def dobldobl_ismember(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a *point* to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the string *point*,
    which is the string representation of a solution in PHCpack format,
    with symbols of the variables before the values of the coordinates.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in double double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    Returns a tuple of two booleans.  The first boolean is True if the
    point satisfies the equations, otherwise it is False.  The second
    boolean is True if the point belongs to the witness set,
    otherwise, the second boolean is False.
    """
    from phcpy.interface import store_dobldobl_witness_set
    from phcpy.phcpy2c3 import py2c_witset_dobldobl_ismember as membtest
    store_dobldobl_witness_set(len(wsys), dim, wsys, gpts)
    nbc = len(point)
    nvr = len(wsys) - dim # test point should not have slack variables
    if verbose:
        print('calling dobldobl_ismember with test point :')
        print(point)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, point)
    (fail, onpolsys, inwitset) = result
    return (onpolsys, inwitset)

def quaddobl_ismember(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a *point* to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the string *point*,
    which is the string representation of a solution in PHCpack format,
    with symbols of the variables before the values of the coordinates.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in quad double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    Returns a tuple of two booleans.  The first boolean is True if the
    point satisfies the equations, otherwise it is False.  The second
    boolean is True if the point belongs to the witness set,
    otherwise, the second boolean is False.
    """
    from phcpy.interface import store_quaddobl_witness_set
    from phcpy.phcpy2c3 import py2c_witset_quaddobl_ismember as membtest
    store_quaddobl_witness_set(len(wsys), dim, wsys, gpts)
    nbc = len(point)
    nvr = len(wsys) - dim # test point should not have slack variables
    if verbose:
        print('calling quaddobl_ismember with test point :')
        print(point)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, point)
    (fail, onpolsys, inwitset) = result
    return (onpolsys, inwitset)

def standard_laurent_ismember(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a *point* to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    Laurent system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the string *point*,
    which is the string representation of a solution in PHCpack format,
    with symbols of the variables before the values of the coordinates.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in standard double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    Returns a tuple of two booleans.  The first boolean is True if the
    point satisfies the equations, otherwise it is False.  The second
    boolean is True if the point belongs to the witness set,
    otherwise, the second boolean is False.
    """
    from phcpy.interface import store_standard_laurent_witness_set
    from phcpy.phcpy2c3 \
    import py2c_witset_standard_Laurent_ismember as membtest
    store_standard_laurent_witness_set(len(wsys), dim, wsys, gpts)
    nbc = len(point)
    nvr = len(wsys) - dim # test point should not have slack variables
    if verbose:
        print('calling standard_ismember with test point :')
        print(point)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, point)
    (fail, onpolsys, inwitset) = result
    return (onpolsys, inwitset)

def dobldobl_laurent_ismember(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a *point* to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    Laurent system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the string *point*,
    which is the string representation of a solution in PHCpack format,
    with symbols of the variables before the values of the coordinates.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in double double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    Returns a tuple of two booleans.  The first boolean is True if the
    point satisfies the equations, otherwise it is False.  The second
    boolean is True if the point belongs to the witness set,
    otherwise, the second boolean is False.
    """
    from phcpy.interface import store_dobldobl_laurent_witness_set
    from phcpy.phcpy2c3 \
    import py2c_witset_dobldobl_Laurent_ismember as membtest
    store_dobldobl_laurent_witness_set(len(wsys), dim, wsys, gpts)
    nbc = len(point)
    nvr = len(wsys) - dim # test point should not have slack variables
    if verbose:
        print('calling dobldobl_ismember with test point :')
        print(point)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, point)
    (fail, onpolsys, inwitset) = result
    return (onpolsys, inwitset)

def quaddobl_laurent_ismember(wsys, gpts, dim, point, \
    evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Applies the homotopy membership test for a *point* to belong to
    a witness set of dimension *dim*, given by an embedding polynomial
    Laurent system in *wsys*, with corresponding generic points in *gpts*.
    The coordinates of the test point are given in the string *point*,
    which is the string representation of a solution in PHCpack format,
    with symbols of the variables before the values of the coordinates.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in quad double precision.
    The default values for the evaluation (*evatol*) and the membership
    (*memtol*) allow for singular values at the end points of the paths
    in the homotopy membership test.
    Returns a tuple of two booleans.  The first boolean is True if the
    point satisfies the equations, otherwise it is False.  The second
    boolean is True if the point belongs to the witness set,
    otherwise, the second boolean is False.
    """
    from phcpy.interface import store_quaddobl_laurent_witness_set
    from phcpy.phcpy2c3 \
    import py2c_witset_quaddobl_Laurent_ismember as membtest
    store_quaddobl_laurent_witness_set(len(wsys), dim, wsys, gpts)
    nbc = len(point)
    nvr = len(wsys) - dim # test point should not have slack variables
    if verbose:
        print('calling quaddobl_ismember with test point :')
        print(point)
    result = membtest(int(verbose), tasks, nvr, dim, nbc, evatol, memtol, point)
    (fail, onpolsys, inwitset) = result
    return (onpolsys, inwitset)

def standard_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Given in *wsys* and *gpts* is a witness set of dimension *dim*,
    where *wsys* is an embedded polynomial system,
    and in *points* a list of strings.  The strings represent points
    as solutions in PHCpack format.  The homotopy membership test is
    applied to each point in the list *points*.  The list on return
    contains the points that do NOT belong to the witness set.
    Points that belong to the witness set are considered junk.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in standard double precision.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    """
    from phcpy.solutions import diagnostics
    result = []
    for point in points:
        rco = diagnostics(point)[1]
        if rco > rcotol:
            (isgood, ismember) = (True, False)
        else:
            tst = standard_ismember(wsys, gpts, dim, point, \
                                    evatol, memtol, verbose, tasks)
            (isgood, ismember) = tst
        if isgood and not ismember:
            result.append(point)
    return result

def dobldobl_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Given in *wsys* and *gpts* is a witness set of dimension *dim*,
    where *wsys* is an embedded polynomial system,
    and in *points* a list of strings.  The strings represent points
    as solutions in PHCpack format.  The homotopy membership test is
    applied to each point in the list *points*.  The list on return
    contains the points that do NOT belong to the witness set.
    Points that belong to the witness set are considered junk.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in double double precision.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    """
    from phcpy.solutions import diagnostics
    result = []
    for point in points:
        rco = diagnostics(point)[1]
        if rco > rcotol:
            (isgood, ismember) = (True, False)
        else:
            tst = dobldobl_ismember(wsys, gpts, dim, point, \
                                    evatol, memtol, verbose, tasks)
            (isgood, ismember) = tst
        if isgood and not ismember:
            result.append(point)
    return result

def quaddobl_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Given in *wsys* and *gpts* is a witness set of dimension *dim*,
    where *wsys* is an embedded polynomial system,
    and in *points* a list of strings.  The strings represent points
    as solutions in PHCpack format.  The homotopy membership test is
    applied to each point in the list *points*.  The list on return
    contains the points that do NOT belong to the witness set.
    Points that belong to the witness set are considered junk.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in quad double precision.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    """
    from phcpy.solutions import diagnostics
    result = []
    for point in points:
        rco = diagnostics(point)[1]
        if rco > rcotol:
            (isgood, ismember) = (True, False)
        else:
            tst = quaddobl_ismember(wsys, gpts, dim, point, \
                                    evatol, memtol, verbose, tasks)
            (isgood, ismember) = tst
        if isgood and not ismember:
            result.append(point)
    return result

def standard_laurent_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Given in *wsys* and *gpts* is a witness set of dimension *dim*,
    where *wsys* is an embedded Laurent polynomial system,
    and in *points* a list of strings.  The strings represent points
    as solutions in PHCpack format.  The homotopy membership test is
    applied to each point in the list *points*.  The list on return
    contains the points that do NOT belong to the witness set.
    Points that belong to the witness set are considered junk.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in standard double precision.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    """
    from phcpy.solutions import diagnostics
    result = []
    for point in points:
        rco = diagnostics(point)[1]
        if rco > rcotol:
            (isgood, ismember) = (True, False)
        else:
            tst = standard_laurent_ismember(wsys, gpts, dim, point, \
                                            evatol, memtol, verbose, tasks)
            (isgood, ismember) = tst
        if isgood and not ismember:
            result.append(point)
    return result

def dobldobl_laurent_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Given in *wsys* and *gpts* is a witness set of dimension *dim*,
    where *wsys* is an embedded Laurent polynomial system,
    and in *points* a list of strings.  The strings represent points
    as solutions in PHCpack format.  The homotopy membership test is
    applied to each point in the list *points*.  The list on return
    contains the points that do NOT belong to the witness set.
    Points that belong to the witness set are considered junk.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in double double precision.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    """
    from phcpy.solutions import diagnostics
    result = []
    for point in points:
        rco = diagnostics(points)[1]
        if rco > rcotol:
            (isgood, ismember) = (True, False)
        else:
            tst = dobldobl_laurent_ismember(wsys, gpts, dim, point, \
                                            evatol, memtol, verbose, tasks)
            (isgood, ismember) = tst
        if isgood and not ismember:
            result.append(point)
    return result

def quaddobl_laurent_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, verbose=True, tasks=0):
    r"""
    Given in *wsys* and *gpts* is a witness set of dimension *dim*,
    where *wsys* is an embedded Laurent polynomial system,
    and in *points* a list of strings.  The strings represent points
    as solutions in PHCpack format.  The homotopy membership test is
    applied to each point in the list *points*.  The list on return
    contains the points that do NOT belong to the witness set.
    Points that belong to the witness set are considered junk.
    By default, *verbose* is True.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    Calculations happen in quad double precision.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    """
    from phcpy.solutions import diagnostics
    result = []
    for point in points:
        rco = diagnostics(point)[1]
        if rco > rcotol:
            (isgood, ismember) = (True, False)
        else:
            tst = quaddobl_laurent_ismember(wsys, gpts, dim, point, \
                                            evatol, memtol, verbose, tasks)
            (isgood, ismember) = tst
        if isgood and not ismember:
            result.append(point)
    return result

def ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, \
    verbose=True, precision='d', tasks=0):
    r"""
    Filters *points* so the list on return contains only those points
    which do not belong to the witness set of dimension *dim*,
    given by an embedded polynomial system in *wsys*,
    with corresponding generic points in *gpts*.
    The list *points* is a list of strings.  Each string is the
    symbolic string representation of a solution.
    By default, *verbose* is True, and the precision is double 'd'.
    Other levels of precision are double double precision 'dd' 
    and quad double precision 'qd'.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    The homotopy membership test has two tolerances: *evatol* and *memtol*.
    The *evatol* is the tolerance on the residual
    of the evaluation of the polynomial equations at the test point.
    If the residual of the evaluation is not less than *evatol*,
    then the point is not a member.  Otherwise, the homotopy
    membership test is called and the *memtol* is used to compare
    the coordinates of the point with the newly computed generic points.
    If there is a match between the coordinates within the given
    tolerance *memtol*, then the points is a member and filtered out.
    """
    if(precision == 'd'):
        return standard_ismember_filter\
                   (wsys, gpts, dim, points, rcotol, evatol, memtol, \
                    verbose, tasks)
    elif(precision == 'dd'):
        return dobldobl_ismember_filter\
                   (wsys, gpts, dim, points, rcotol, evatol, memtol, \
                    verbose, tasks)
    elif(precision == 'qd'):
        return quaddobl_ismember_filter\
                   (wsys, gpts, dim, points, rcotol, evatol, memtol, \
                    verbose, tasks)
    else:
        print('wrong argument for precision')
        return points

def laurent_ismember_filter(wsys, gpts, dim, points, \
    rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, \
    verbose=True, precision='d', tasks=0):
    r"""
    Filters *points* so the list on return contains only those points
    which do not belong to the witness set of dimension *dim*,
    given by an embedded Laurent polynomial system in *wsys*,
    with corresponding generic points in *gpts*.
    The list *points* is a list of strings.  Each string is the
    symbolic string representation of a solution.
    By default, *verbose* is True, and the precision is double 'd'.
    Other levels of precision are double double precision 'dd' 
    and quad double precision 'qd'.
    The number of threads is given in *tasks*.  If *tasks* is zero,
    then no multithreading is applied in the homotopy membership test.
    The parameter *rcotol* is used to bypass the homotopy membership test,
    for points with their estimated inverse condition number larger than
    *rcotol* will be considered isolated and not in the witness set.
    The homotopy membership test has two tolerances: *evatol* and *memtol*.
    The *evatol* is the tolerance on the residual
    of the evaluation of the polynomial equations at the test point.
    If the residual of the evaluation is not less than *evatol*,
    then the point is not a member.  Otherwise, the homotopy
    membership test is called and the *memtol* is used to compare
    the coordinates of the point with the newly computed generic points.
    If there is a match between the coordinates within the given
    tolerance *memtol*, then the point is a member and filtered out.
    """
    if(precision == 'd'):
        return standard_laurent_ismember_filter\
                   (wsys, gpts, dim, points, rcotol, evatol, memtol,
                    verbose, tasks)
    elif(precision == 'dd'):
        return dobldobl_laurent_ismember_filter\
                   (wsys, gpts, dim, points, rcotol, evatol, memtol,
                    verbose, tasks)
    elif(precision == 'qd'):
        return quaddobl_laurent_ismember_filter\
                   (wsys, gpts, dim, points, rcotol, evatol, memtol,
                    verbose, tasks)
    else:
        print('wrong argument for precision')
        return points

def is_slackvar(var):
    r"""
    Given in *var* is a string with a variable name.
    Returns True if the variable name starts with 'zz',
    followed by a number.  Returns False otherwise.
    """
    if len(var) <= 2: # too short to be a slack variable
        return False
    elif var[:2] != 'zz': # a slack variable starts with zz
        return False
    else:
        rest = var[2:]
        return rest.isdigit()

def is_signed(pol):
    r"""
    Given in *pol* is the string representation of a polynomial.
    Returns True if the first nonspace character in the string *pol*
    is either '+' or '-'.  Returns False otherwise.
    """
    wrk = pol.lstrip()
    if len(wrk) == 0: # all characters in pol are spaces
        return False
    else:
        return (wrk[0] == '+' or wrk[0] == '-')

def is_member(wsys, gpts, dim, solpt, evatol=1.0e-6, memtol=1.0e-6, \
    verbose=True, precision='d', tasks=0):
    r"""
    This function wraps the membertest where the point is a solution,
    given in *solpt*.  All other parameters have the same meaning as
    in the function membertest.
    """
    from phcpy.solutions import strsol2dict, variables
    from phcpy.solver import names_of_variables
    soldc = strsol2dict(solpt)
    svars = variables(soldc)
    pvars = names_of_variables(wsys)
    if verbose:
        print('variables in solution :', svars)
        print('variables in system   :', pvars)
    (ordvar, solval) = ('', [])
    for var in svars:
        if not is_slackvar(var): # skip the slack variables
            if var not in pvars:
                print('Error: mismatch of variables names.')
                return None
            ordvar = ordvar + '+' + var + '-' + var
            solval.append(soldc[var].real)
            if precision == 'dd':
                solval.append(0.0)
            elif precision == 'qd':
                solval.append(0.0)
                solval.append(0.0)
                solval.append(0.0)
            solval.append(soldc[var].imag)
            if precision == 'dd':
                solval.append(0.0)
            elif precision == 'qd':
                solval.append(0.0)
                solval.append(0.0)
                solval.append(0.0)
    if verbose:
        print('order of the variables :', ordvar)
        print('values in the point :', solval)
    wpol0 = wsys[0]
    if is_signed(wpol0):
        newpol0 = ordvar + wpol0
    else:
        newpol0 = ordvar + '+' + wpol0
    newsys = [newpol0] + wsys[1:]
    return membertest(newsys, gpts, dim, solval, evatol, memtol, \
                      verbose, precision)

def test_member(prc='d'):
    """
    To test the membership, we take the twisted cubic.
    """
    twisted = ['x^2 - y;', 'x^3 - z;']
    twiste1 = embed(3, 1, twisted)
    twiste1[0] = 'x + y + z - x - y - z + ' + twiste1[0]
    from phcpy.solver import solve
    twtsols = solve(twiste1, precision=prc)
    for sol in twtsols:
        print(sol)
    if(prc == 'd'):
        inpoint = [1, 0, 1, 0, 1, 0]
        outpoint = [1, 0, 1, 0, 2, 0]
    elif(prc == 'dd'):
        inpoint = [1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0]
        outpoint = [1, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0]
    elif(prc == 'qd'):
        inpoint = [1, 0, 0, 0, 0, 0, 0, 0, \
                   1, 0, 0, 0, 0, 0, 0, 0, \
                   1, 0, 0, 0, 0, 0, 0, 0]
        outpoint = [1, 0, 0, 0, 0, 0, 0, 0, \
                    1, 0, 0, 0, 0, 0, 0, 0, \
                    2, 0, 0, 0, 0, 0, 0, 0, ]
    print(membertest(twiste1, twtsols, 1, inpoint, precision=prc))
    print(membertest(twiste1, twtsols, 1, outpoint, precision=prc))

def test_twisted_ismember(prc='d', laurent=True):
    """
    To test the membertest wrapper, we take the twisted cubic again.
    The test point is given as a solution in PHCpack format.
    """
    from phcpy.solver import solve
    from phcpy.solutions import make_solution, strsol2dict
    twisted = ['x^2 - y;', 'x^3 - z;']
    twiste1 = embed(3, 1, twisted)
    twtsols = solve(twiste1, precision=prc)
    for sol in twtsols:
        print(sol)
    input('hit enter to continue')
    # print(is_member(twiste1, twtsols, 1, twtsols[0], precision=prc))
    genpt = twtsols[0]
    dicpt = strsol2dict(genpt)
    coord = [dicpt['x'], dicpt['y'], dicpt['z']]
    tstpt = make_solution(['x', 'y', 'z'], coord)
    if laurent:
        if prc == 'd':
            print(standard_laurent_ismember\
                      (twiste1, twtsols, 1, tstpt, tasks=3))
        elif prc == 'dd':
            print(dobldobl_laurent_ismember\
                      (twiste1, twtsols, 1, tstpt, tasks=3))
        elif prc == 'qd':
            print(quaddobl_laurent_ismember\
                      (twiste1, twtsols, 1, tstpt, tasks=3))
        else:
            print( 'wrong level of precision')
    else:
        if prc == 'd':
            print(standard_ismember(twiste1, twtsols, 1, tstpt, tasks=3))
        elif prc == 'dd':
            print(dobldobl_ismember(twiste1, twtsols, 1, tstpt, tasks=3))
        elif prc == 'qd':
            print(quaddobl_ismember(twiste1, twtsols, 1, tstpt, tasks=3))
        else:
            print( 'wrong level of precision')
    input('hit enter to continue')
    outsol = make_solution(['x', 'y', 'z'], [1, 2, 3])
    # print(is_member(twiste1, twtsols, 1, outsol, precision=prc))
    if laurent:
        if prc == 'd':
            print(standard_laurent_ismember\
                      (twiste1, twtsols, 1, outsol, tasks=3))
        elif prc == 'dd':
            print(dobldobl_laurent_ismember\
                      (twiste1, twtsols, 1, outsol, tasks=3))
        elif prc == 'qd':
            print(quaddobl_laurent_ismember\
                      (twiste1, twtsols, 1, outsol, tasks=3))
        else:
            print('wrong level of precision')
    else:
        if prc == 'd':
            print(standard_ismember(twiste1, twtsols, 1, outsol, tasks=3))
        elif prc == 'dd':
            print(dobldobl_ismember(twiste1, twtsols, 1, outsol, tasks=3))
        elif prc == 'qd':
            print(quaddobl_ismember(twiste1, twtsols, 1, outsol, tasks=3))
        else:
            print('wrong level of precision')

def test():
    """
    Runs a test on algebraic sets.
    """
    from phcpy.phcpy2c3 import py2c_set_seed
    py2c_set_seed(234798272)
    test_twisted_ismember(prc='qd', laurent=False)

if __name__ == "__main__":
    test()



"""
This script computes the tangents lines to a circle
via a witness set intersection, with phcpy in Python 3.
With matplotlib a plot is made of the tangent lines.
"""
from math import cos, sin, pi
from random import uniform
import matplotlib.pyplot as plt
from phcpy.solver import solve
from phcpy.solutions import make_solution, coordinates, strsol2dict
from phcpy.sets import double_embed
from phcpy.diagonal import double_diagonal_solve

def polynomials(apt, bpt, rad):
    """
    Returns string representations of two polynomials:
    1) a circle with radius r centered at (apt, bpt);
    2) a line through the origin with slope s.
    """
    crc = f'(x - {apt:.15e})^2 + (y - {bpt:.15e})^2 - {rad**2:.15e};'
    lin = 'y - s*x;'
    return [crc, lin]

def special_solutions(pols, slope):
    """
    Given in pols the polynomials for the line intersecting a circle
    for a general slope s, solves the problem for a special numerical
    value for s, given in the slope.
    The value of the slope is added as last coordinate to each solution.
    """
    special = []
    for pol in pols:
        rpl = pol.replace('s', '(' + str(slope) + ')')
        special.append(rpl)
    sols = solve(special)
    result = []
    for sol in sols:
        (names, values) = coordinates(sol)
        names.append('s')
        values.append(slope)
        extsol = make_solution(names, values)
        result.append(extsol)
    return result

def make_witness_set(pols, verbose=True):
    """
    We have two equations in three variables in pols
    and therefore we expect a one dimensional set.
    """
    embpols = double_embed(3, 1, pols)
    if verbose:
        print('the embedded system :')
        for pol in embpols:
            print(pol)
    embsols = solve(embpols)
    if verbose:
        print('the witness points :')
        for sol in embsols:
            print(sol)
    return (embpols, embsols)

def circle_line_set():
    """
    Generates the system and its witness set.
    Returns the witness set for a fixed circle 
    intersect with a one parameter family of lines 
    as a tuple of polynomials and solutions
    """
    syst = polynomials(3, 2, 1)
    for pol in syst:
        print(pol)
    spsols = special_solutions(syst, 1)
    for sol in spsols:
        print(sol)
    input('hit enter to continue')
    (embsyst, embsols) = make_witness_set(syst, False)
    print('the polynomials in the witness set:')
    for pol in embsyst:
        print(pol)
    print('the solutions :')
    for sol in embsols:
        print(sol)
    print('degree of the set :', len(embsols))
    return (embsyst, embsols)

def random_complex():
    """
    Returns a random complex number on the unit circle.
    """
    theta = uniform(0, 2*pi)
    return complex(cos(theta), sin(theta))

def random_hyperplane(variables):
    """
    Returns a linear equation in the variables,
    with random complex coefficients.
    """
    cf0 = str(random_complex())
    tf0 = cf0.replace('j', '*i')
    result = tf0
    for var in variables:
        cff = str(random_complex())
        tcf = cff.replace('j', '*i')
        result = result + '+' + tcf + '*' + var
    return result + ';'

def jacobian(apt, bpt):
    """
    Returns the equations which define the points
    where the Jacobian matrix is singular,
    as a random linear combination of the columns.
    Random complex coefficients are generated to
    scale the multiplier variables.
    """
    eq1 = f'2*(x-{apt:.15e})*L1 + 2*(y-{bpt:.15e})*L2;'
    eq2 = '-s*L1 + L2;'
    eq3 = random_hyperplane(['L1', 'L2'])
    print('eq3 = ', eq3)
    return [eq1, eq2, eq3]

def extend_solutions(sols):
    """
    To each solution in sols, adds L1 and L2 with values 1,
    and zz2 and zz3 with values zero.
    """
    result = []
    for sol in sols:
        (names, values) = coordinates(sol)
        names = names + ['L1', 'L2', 'zz2', 'zz3']
        values = values + [1, 1, 0, 0]
        extsol = make_solution(names, values)
        result.append(extsol)
    return result

def extend(pols, sols, verbose=True):
    """
    Extends the witness set with two free variables
    L1 and L2, addition two linear equations,
    and two slack variables zz2 and zz3.
    """
    eq1 = 'zz2;'
    eq2 = 'zz3;'
    eq3 = 'L1 - 1;'
    eq4 = 'L2 - 1;'
    extpols = pols[:-1] + [eq1, eq2, eq3, eq4, pols[-1]]
    extsols = extend_solutions(sols)
    if verbose:
        print('the extended polynomials :')
        for pol in extpols:
            print(pol)
        input('hit enter to continue')
        print('the extended solutions :')
        for sol in extsols:
            print(sol)
        input('hit enter to continue')
    return (extpols, extsols)

def witset(pols, verbose=True):
    """
    We have three equations in pols in five variables:
    x, y, s, L1, and L2.
    Therefore we expect a two dimensional set.
    """
    embpols = double_embed(5, 2, pols)
    if verbose:
        print('the embedded system :')
        for pol in embpols:
            print(pol)
    embsols = solve(embpols)
    if verbose:
        print('the witness points :')
        for sol in embsols:
            print(sol)
    return (embpols, embsols)

def insert_symbols(pol):
    """
    To the string pol, adds the sequence of symbols.
    """
    qpol = pol.lstrip()
    if qpol[0] == '+' or qpol[0] == '-':
        smb = 'x - x + y - y + s + L1 - L1 + L2 - L2 - s '
    else:
        smb = 'x - x + y - y + s + L1 - L1 + L2 - L2 - s + '
    return smb + pol

def intersect(dim, w1d, w2d, ws1, ws2):
    """
    Applies the diagonal homotopy to intersect two witness sets
    w1 and w2 of dimensions w1d and w2d in a space of dimension dim.
    """
    w1eqs, w1sols = ws1
    w2eqs, w2sols = ws2
    nw1eq0 = insert_symbols(w1eqs[0])
    nw2eq0 = insert_symbols(w2eqs[0])
    nw1eqs = [nw1eq0] + w1eqs[1:]
    nw2eqs = [nw2eq0] + w2eqs[1:]
    print('number of equations in first witness set :', len(w1eqs))
    print('number of equations in second witness set :', len(w2eqs))
    ans = input('Verbose version of diagonal solve ? (y/n) ')
    otp = int(ans == 'y')
    result = double_diagonal_solve(dim, w1d, nw1eqs, w1sols, w2d, nw2eqs, w2sols, \
        vrblvl=otp)
    (eqs, sols) = result
    print('the equations :')
    for pol in eqs:
        print(pol)
    input('hit enter to continue')
    print('the solutions :')
    for sol in sols:
        print(sol)
    print('computed', len(sols), 'solutions')
    input('hit enter to continue')
    return result

def singular_locus_set():
    """
    Generates a witness set for the singular locus
    of the algebraic set of a fixed circle intersected
    with a one parameter family of lines.
    """
    syst = jacobian(3, 2)
    for pol in syst:
        print(pol)
    (embsyst, embsols) = witset(syst, False)
    print('the polynomials in the witness set :')
    for pol in embsyst:
        print(pol)
    input('hit enter to continue')
    print('the solutions :')
    for sol in embsols:
        print(sol)
    print('degree of the singular locus set :', len(embsols))
    input('hit enter to continue')
    return (embsyst, embsols)

def coordinates_and_slopes(sol):
    """
    Given a solution, return the 3-tuple with the x and y coordinates
    of the tangent point and the slope of the tangent line.
    The real parts of the coordinates are selected.
    """
    sdc = strsol2dict(sol)
    return (sdc['x'].real, sdc['y'].real, sdc['s'].real)

def plotgeneral(fig):
    """
    Plots the circle with radius 1, centered at (3, 2),
    along with a line with slope one through (0, 0).
    """
    axs = fig.gca()
    center = (3, 2)
    radius = 1
    circle = plt.Circle(center, radius, edgecolor='blue', \
       facecolor='none')
    axs.add_artist(circle)
    plt.plot([0, 4], [0, 4], 'r')
    plt.plot([2, 3], [2, 3], 'go')
    plt.axis([0, 5, 0, 4])

def plotspecial(fig, sol1, sol2):
    """
    Plots the circle with radius 1, centered at (3, 2),
    along with its two lines tangent through (0, 0).
    The two solutions sol1 and sol2 define 3-tuples of
    x and y coordinates and a slope.
    """
    (xp1, yp1, slope1) = sol1
    (xp2, yp2, slope2) = sol2
    axs = fig.gca()
    center = (3, 2)
    radius = 1
    circle = plt.Circle(center, radius, edgecolor='blue', \
       facecolor='none')
    axs.add_artist(circle)
    y1r = 5*slope1
    y2r = 5*slope2
    plt.plot([0, 5], [0, y1r], 'r')
    plt.plot([0, 5], [0, y2r], 'r')
    plt.plot([xp1, xp2], [yp1, yp2], 'go')
    plt.axis([0, 5, 0, 4])

def show_solutions(sol1, sol2):
    """
    Makes a matplotlib figure with a general configuration
    and the tangent lines defined by sol1 and sol2.
    """
    fig = plt.figure()
    fig.add_subplot(121, aspect='equal')
    plotgeneral(fig)
    fig.add_subplot(122, aspect='equal')
    plotspecial(fig, sol1, sol2)
    fig.show()
    input('hit enter to exit')

def main():
    """
    Defines two witness sets and intersects them.
    """
    (witpols, witsols) = circle_line_set()
    input('hit enter to continue')
    witset1 = extend(witpols, witsols)
    witset2 = singular_locus_set()
    intwitset = intersect(5, 3, 2, witset1, witset2)
    (eqs, sols) = intwitset
    print('the equations :')
    for equ in eqs:
        print(equ)
    print('the solutions :')
    for (idx, sol) in enumerate(sols):
        print('Solution', idx+1, ':')
        print(sol)
    sol1 = coordinates_and_slopes(sols[0])
    sol2 = coordinates_and_slopes(sols[1])
    show_solutions(sol1, sol2)

if __name__ == "__main__":
    main()



"""
The circle problem of Apollonius has the following input/output specification:
Given three circles, find all circles that are tangent to the given circles.
Without loss of generality, we take the first circle to be the unit circle,
centered at (0, 0) and with radius 1.  The origin of the second circle lies
on the first coordinate axis, so its center has coordinates (c2x, 0) and
radius r2.  The third circle has center (c3x, c3y) and radius r3.
So there are five parameters in this problem: c2x, r2, c3x, c3y, and r3.
Values for the five parameters are defined by the first five equations.
The next three equations determine the center (x, y) and the radius r
of the circle which touches the three given circles.
The condition on the center of the touching circle is that its distance
to the center of the given circle is either the difference or the sum of
the radii of both circles.  So we arrive at eight polynomial systems.
This Python 3 script solves the systems with phcpy and then prints those
solution circles that have a positive radius.
With matplotlib, the given circles and the solutions are plotted.
"""
def polynomials(c2x, r2, c3x, c3y, r3):
    """
    On input are the five parameters of the circle problem of Apollonius:
    c2x : the x-coordinate of the center of the second circle,
    r2 : the radius of the second circle,
    c3x : the x-coordinate of the center of the third circle,
    c3y : the y-coordinate of the center of the third circle,
    r3 : the radius of the third circle.
    Returns a list of lists.  Each list contains a polynomial system.
    Solutions to each polynomial system define center (x, y) and radius r
    of a circle touching three given circles.
    """
    e1m = 'x^2 + y^2 - (r-1)^2;'
    e1p = 'x^2 + y^2 - (r+1)^2;'
    e2m = '(x-%.15f)^2 + y^2 - (r-%.15f)^2;' % (c2x, r2)
    e2p = '(x-%.15f)^2 + y^2 - (r+%.15f)^2;' % (c2x, r2)
    e3m = '(x-%.15f)^2 + (y-%.15f)^2 - (r-%.15f)^2;' % (c3x, c3y, r3)
    e3p = '(x-%.15f)^2 + (y-%.15f)^2 - (r+%.15f)^2;' % (c3x, c3y, r3)
    eqs0 = [e1m,e2m,e3m]
    eqs1 = [e1m,e2m,e3p]
    eqs2 = [e1m,e2p,e3m]
    eqs3 = [e1m,e2p,e3p]
    eqs4 = [e1p,e2m,e3m]
    eqs5 = [e1p,e2m,e3p]
    eqs6 = [e1p,e2p,e3m]
    eqs7 = [e1p,e2p,e3p]
    return [eqs0,eqs1,eqs2,eqs3,eqs4,eqs5,eqs6,eqs7]

def solve4circles(syst, verbose=True):
    """
    Given in syst is a list of polynomial systems.
    Returns a list of tuples.  Each tuple in the list of return
    consists of the coordinates of the center and the radius of
    a circle touching the three given circles.
    """
    from phcpy.solver import solve
    from phcpy.solutions import strsol2dict, is_real
    (circle, eqscnt) = (0, 0)
    result = []
    for eqs in syst:
        eqscnt = eqscnt + 1
        if verbose:
            print('solving system', eqscnt, ':')
            for pol in eqs:
                print(pol)
        sols = solve(eqs, verbose=False)
        if verbose:
            print('system', eqscnt, 'has', len(sols), 'solutions')
        for sol in sols:
            if is_real(sol, 1.0e-8):
                soldic = strsol2dict(sol)
                if soldic['r'].real > 0:
                    circle = circle + 1
                    ctr = (soldic['x'].real, soldic['y'].real)
                    rad = soldic['r'].real
                    result.append((ctr, rad))
                    if verbose:
                        print('solution circle', circle)
                        print('center =', ctr)
                        print('radius =', rad)
    return result

def makecircles(plt, data, color, disk=False, verbose=True):
    """
    Gives in data a list of centers and radii,
    returns a list of matplotlib objects, using the color as edgecolor,
    The pyplot in matplotlib is passed as plt.
    """
    result = []
    for (center, radius) in data:
        if verbose:
            print('circle with center', center, 'and radius', radius)
        if disk:
            crc = plt.Circle(center, radius, edgecolor=color, \
                 facecolor=color) #, linewidth=2)
        else:
            crc = plt.Circle(center, radius, edgecolor=color, \
                 facecolor='none') # , linewidth=2)
        result.append(crc)
    return result

def plotcircles(plt, circles, xa, xb, ya, yb):
    """
    Given on input in circles a list of matplotlib objects,
    the circles are rendered in a figure.
    The pyplot in matplotlib is passed as plt.
    The range of the axis in the matplotlib plot is defined
    by the four numbers xa, xb, ya, and yb.
    """
    fig = plt.figure()
    fig.add_subplot(111, aspect='equal')
    axs = plt.gcf().gca()
    for circle in circles:
        axs.add_artist(circle)
    plt.axis([xa, xb, ya, yb])
    fig.show() # plt.show()
    ans = input('hit enter to continue')

def solve_general_problem():
    """
    Solves a general configuration of three circles.
    Defines a list of polynomial systems, solves each system
    and extract those real solutions with positive radius.
    The given circles are plotted as blue disks,
    while the eight solution circles are plotted in red.
    """
    syst = polynomials(2, 2.0/3, 1, 1, 1.0/3)
    sols = solve4circles(syst)
    print('the solution list :')
    print(sols)
    ans = input('Continue with matplotlib ? (y/n) ')
    if ans == 'y':
        import matplotlib.pyplot as plt
        crcdata = [((0, 0), 1), ((2, 0), 2.0/3), ((1, 1), 1.0/3)]
        incircles = makecircles(plt, crcdata, disk=True, color='blue')
        outcircles = makecircles(plt, sols, color='red')
        plotcircles(plt, incircles + outcircles, -3, 11, -3, 8)

def solve_special_problem():
    """
    Solves a special configuration of three circles,
    where the three circles are mutually touching each other.
    Defines a list of polynomial systems, solves each system
    and extract those real solutions with positive radius.
    The given circles are plotted as blue disks,
    while the solution circles are plotted in red.
    """
    from math import sqrt
    height = sqrt(3)
    syst = polynomials(2, 1, 1, height, 1)
    sols = solve4circles(syst)
    print('the solution list :')
    print(sols)
    ans = input('Continue with matplotlib ? (y/n) ')
    if ans == 'y':
        import matplotlib.pyplot as plt
        crcdata = [((0, 0), 1), ((2, 0), 1), ((1, height), 1)]
        incircles = makecircles(plt, crcdata, disk=True, color='blue')
        outcircles = makecircles(plt, sols, color='red')
        plotcircles(plt, incircles + outcircles, -2, 4, -2, 3)

def solve_perturbed_problem():
    """
    Solves a small perturbation of a special configuration of three circles,
    where the three circles are mutually touching each other.
    Defines a list of polynomial systems, solves each system
    and extract those real solutions with positive radius.
    The given circles are plotted as blue disks,
    while the solution circles are plotted in red.
    """
    from math import sqrt
    height = sqrt(3)
    syst = polynomials(2.05, 1, 1.025, height+0.025, 1)
    sols = solve4circles(syst)
    print('the solution list :')
    print(sols)
    ans = input('Continue with matplotlib ? (y/n) ')
    if ans == 'y':
        import matplotlib.pyplot as plt
        crcdata = [((0, 0), 1), ((2.05, 0), 1), ((1.025, height+0.025), 1)]
        incircles = makecircles(plt, crcdata, disk=True, color='blue')
        outcircles = makecircles(plt, sols, color='red')
        plotcircles(plt, incircles + outcircles, -2, 4, -2, 4)

def main():
    """
    Solves a general and a special instance of the circle problem
    of Apollonius.
    """
    print('solving a general instance of the Apollonius circle problem')
    solve_general_problem()
    print('solving a special instance of the Apollonius circle problem')
    solve_special_problem()
    print('solving a perturbed instance of the Apollonius circle problem')
    solve_perturbed_problem()

if __name__=="__main__":
    main()



import threading
import Queue
from subprocess import PIPE, Popen
from phc_config import client_id_len, phc_cmd, phc_python, phc_maple
from phc_phc import PHC_Kill, processes
from phc_file import get_file_id, get_file_pid
import os

class solver_local(threading.Thread):
   def __init__(self, queue, Usersolver, name):
      threading.Thread.__init__(self)
      self.queue = queue
      self.Usersolver = Usersolver
      self.name = name
      # self.queue_file = queue_file

   #----------------------------------------------------------------------
   def run(self):
      while True:
         # gets the polynomial system from the queue
         [opt, Uid, msg] = self.queue.get()
         if opt == '1':
            # solves the polynomial system
            self.polysolver(Uid, msg)
            self.queue.task_done()
         elif opt == '2':
            # kill polynomial
            self.phc_kill(Uid,msg)
            self.queue.task_done()
         elif opt == '6':
            # generate print different formats
            self.phcprint(msg)
            self.queue.task_done()
         elif opt == '7':
            # generate homotopy
            self.phc_homotopy(Uid,msg)
            self.queue.task_done()


   #----------------------------------------------------------------------
   def polyfile(self, Uid, msg, poly_id):
      from phc_file import rm_exts
      file_id = "../users/%s/%s" %(Uid, poly_id)
      
      exts = ['cmd','sol','phc','err']
      rm_exts(file_id,exts)
      
      with open( file_id+'.sol', 'w') as f:
         f.write(msg)
         f.close()
      
      return file_id

   #----------------------------------------------------------------------
   def polysolver(self, Uid, msg, poly_id = "current"):
      file_id = self.polyfile(Uid, msg, poly_id)
      #cmd = phc_cmd + " %s.sol %s.phc > %s.outfile &" %(file_id, file_id, file_id)
      #print cmd
      #proc = Popen(cmd, stdout=PIPE, shell=True, preexec_fn=os.setsid)

      cmd = phc_cmd + ['%s.sol'%file_id, '%s.phc'%file_id]
      print 'cmd = %s'%cmd
      outf = open('%s.outfile'%file_id,'w')
      proc = Popen(cmd, stdout=outf)

      Pid = str(proc.pid)
      print "Pid = %s"%Pid
      with open('%s.pid'%file_id,'w') as f:
         f.write(Pid)
         f.close()
      return Pid

   #----------------------------------------------------------------------
   def polyfile_homotopy(self, Uid, msg):
      from phc_file import rm_exts, phc_start
      from os import path
      
      [poly_id, poly_id_start, poly_data] = msg.split('|')
      
      # Generate poly file
      file_id = "../users/%s/%s" %(Uid, poly_id)

      exts = ['cmd', 'sol','phc']
      rm_exts(file_id,exts)

      print "msg = %s"% msg
   
      # Generate start system file from phc file         
      file_id_start= "../users/%s/%s" %(Uid, poly_id_start)
      
      rm_exts(file_id_start, ['cmd'])
      
      if not path.isfile(file_id_start+".start"):
         phc_start(file_id_start)
      
      with open('%s.cmd'%file_id_start, 'w') as f:
         f.write("y\n%s.sol\n%s.start\n0\n0\n0\n0\n"%(file_id, file_id_start))
      f.close()
      
      return file_id, file_id_start

   #----------------------------------------------------------------------
   def phc_homotopy(self, Uid, msg):
      """
      Path Tracking by homotypy in 1 parameter
      """ 
      from subprocess import PIPE, Popen
      from os import setsid
      from phc_config import phc_hom

      file_id, file_id_start = self.polyfile_homotopy(Uid, msg)
      cmd = phc_hom + ['%s.poly'%file_id, '%s.phc'%file_id]
      print 'cmd = %s'%cmd
      inf = open('%s.cmd'%file_id_start)
      outf = open('output','w')
      proc = Popen(cmd,stdin=inf,stdout=outf)
      Pid = str(proc.pid)
      print "Pid = %s"%Pid
      with open('%s.pid'%file_id,'w') as f:
         f.write(Pid)
         f.close()

      return Pid

   def phc_kill(self, Uid, msg, poly_id = ''):
      if poly_id == '':
         poly_id = 'current'
      print 'Before kill:\n'
      print processes()
      file_id = get_file_id(Uid, poly_id)
      pid = self.get_pid(Uid, poly_id, file_id)
      if pid !='':
         PHC_Kill(pid, file_id)
         print 'After kill:\n'
         print processes()
      else:
         print "No process to kill. The process has not started."

   def get_pid(self, Uid, poly_id, file_id):
      from os import path
      pid = ''
      job_key = Uid+'/'+poly_id
      if job_key in self.Usersolver:
         pid = self.Usersolver[job_key][1]
         print 'aaa, pid = %d'%pid
      elif path.isfile('%s.pid'%file_id):
         pid = get_file_pid(file_id)
         print 'bbb, pid = %d'%pid
      return pid
      

   #----------------------------------------------------------------------
   def phcprint(self, file_id):
      # send a signal to the queue that the job is done
      cmd = phc_python + " %s.phc > %s.python"% (file_id,file_id)
      s = os.system(cmd)
      cmd = phc_maple + " %s.phc > %s.maple"% (file_id,file_id)
      s = os.system(cmd)



from collections import deque
from phc_config import file_recover

def read_next_symbol(data_str, pos_start, symbol):
   pos_start += 1
   while pos_start < len(data_str):
      if data_str[pos_start] == symbol:
         break;
      pos_start += 1
   return pos_start

def read_element(data_str, symbol = ', '):
   element = []
   data = data_str.split(symbol)
   for i in xrange(len(data)):
      element.append(data[i])
   return element

def read_queue(file_id):
   queue = deque()
   with open(file_id,'r') as f:
      data_str = f.read()
      pos_start = 0
      pos_start = read_next_symbol(data_str,pos_start,'[')
      print pos_start

      while True:
         if data_str[pos_start+1] == ']':
            break;
         pos_start = read_next_symbol(data_str,pos_start,'[')
         pos_end = read_next_symbol(data_str,pos_start,']')
         element = read_element(data_str[pos_start+1:pos_end])
         queue.append(element)
         pos_start = pos_end

   return queue

#read_queue(file_recover)

def recover_queue(file_id = file_recover):
   """
   Recover a queue from a file
   """
   from os import path, remove
   if path.isfile(file_id):
      queue = read_queue(file_id)
      remove(file_id)
      return queue
   else:
      queue = deque()
      return queue

def element2str(element, symbol = ', '):
   """
   Change a list to a string, elements are splitted by symbol
   """
   element_str = '['
   for i in xrange(len(element)-1):
      element_str += element[i]
      element_str += symbol
   element_str +=  element[-1] + ']'
   return element_str

def save_queue(queue, file_id = file_recover):
   """
   Save a queue into a file
   """
   with open(file_id, 'w') as f:
      f.write('queue = [')
      while queue:
         element = queue.popleft()
         f.write(element2str(element))
      f.write(']')
      f.close()



"""
This module allows to work with monomial maps, defined by binomial systems.
"""

def is_binomial_system(silent=True):
    r"""
    Returns True if the system stored in the Laurent systems
    container is a binomial system, returns False otherwise.
    if not *silent*, then the number of terms in each Laurent
    polynomial is written to screen.
    """
    from phcpy.phcpy2c2 import py2c_syscon_number_of_standard_Laurentials
    from phcpy.phcpy2c2 import py2c_syscon_number_of_Laurent_terms
    nbequ = py2c_syscon_number_of_standard_Laurentials()
    if not silent:
        print 'checking if binomial system ...'
        print '  number of Laurent polynomials :', nbequ
    for i in range(1, nbequ+1):
        nbterms = py2c_syscon_number_of_Laurent_terms(i)
        if not silent:
            print '  -> number of terms in polynomial', i, ':', nbterms
        if(nbterms != 2):
            if not silent:
                print '  the system is not a binomial system'
            return False
    if not silent:
        print '  the system is a binomial system'
    return True

def store_laurent_system(nbvar, pols):
    r"""
    Given in *pols* a list of string representing Laurent polynomials
    into the systems container.  The number of variables equals *nbvar*.
    """
    from phcpy.phcpy2c2 import py2c_syscon_clear_standard_Laurent_system
    from phcpy.phcpy2c2 \
        import py2c_syscon_initialize_number_of_standard_Laurentials
    from phcpy.phcpy2c2 import py2c_syscon_store_standard_Laurential
    py2c_syscon_clear_standard_Laurent_system()
    nbequ = len(pols)
    py2c_syscon_initialize_number_of_standard_Laurentials(nbequ)
    for ind in range(0, nbequ):
        pol = pols[ind]
        # print 'storing' , pol
        nbchar = len(pol)
        py2c_syscon_store_standard_Laurential(nbchar, nbvar, ind+1, pol)

def monomial_map_strings(dim, ind, nbvar):
    r"""
    Returns the list of strings representing the components of
    the monomial map of dimension *dim*, with index *ind*, and
    where the number of variables equals *nbvar*.
    """
    from phcpy.phcpy2c2 import py2c_mapcon_coefficients_of_map
    from phcpy.phcpy2c2 import py2c_mapcon_exponents_of_map
    from phcpy.phcpy2c2 import py2c_syscon_string_of_symbols
    sym = py2c_syscon_string_of_symbols()
    symvars = sym.split(' ')
    # py2c_mapcon_write_maps may have inserted t-varables
    thevars = symvars[-nbvar:]
    # print '-> the variables', symvars, thevars
    cff = py2c_mapcon_coefficients_of_map(dim, ind, nbvar)
    # print '-> coefficients of map', ind, ':', cff
    exp = py2c_mapcon_exponents_of_map(dim, ind, nbvar)
    # print '-> exponents of map', ind, ':', exp
    result = []
    exp_ind = 0
    for i in range(0, nbvar):
        str_var = thevars[i]
        if(cff[i] == 0):
            str_var = str_var + ' - 0'
            exp_ind = exp_ind + dim
        else:
            str_var = str_var + ' - ' + str(cff[i])
            for j in range(0, dim):
                pwr = exp[exp_ind]
                if(pwr != 0):
                    str_var = str_var + '*' + 't' + str(j+1) + '**' + str(pwr)
                exp_ind = exp_ind + 1
        result.append(str_var)
    return result

def write_monomial_map(dim, ind, nbvar):
    r"""
    Write the monomial map of dimension *dim* and of index *ind*,
    with number of variables equal to *nbvar*.
    """
    str_map = monomial_map_strings(dim, ind, nbvar)
    print str_map
    for str_var in str_map:
        print str_var

def monomial_map_solutions(nbvar, with_degree=True):
    r"""
    Returns the list of lists of strings,
    each list of strings representing a monomial map
    stored in the container.
    The number of variables equals *nbvar*.
    """
    from phcpy.phcpy2c2 import py2c_mapcon_top_dimension
    from phcpy.phcpy2c2 import py2c_mapcon_number_of_maps
    from phcpy.phcpy2c2 import py2c_mapcon_degree_of_map
    result = []
    topdim = py2c_mapcon_top_dimension()
    for dim in range(topdim, -1, -1):
        nbmaps = py2c_mapcon_number_of_maps(dim)
        for ind in range(1, nbmaps+1):
            str_map = monomial_map_strings(dim, ind, nbvar)
            str_map.append('dimension = %d' % dim)
            if with_degree:
                degmap = py2c_mapcon_degree_of_map(dim, ind)
                str_map.append('degree = %d' % degmap)
            result.append(str_map)
    return result

def write_monomial_maps(nbvar):
    """
    Writes the maps stored in the container.
    """
    from phcpy.phcpy2c2 import py2c_mapcon_top_dimension
    from phcpy.phcpy2c2 import py2c_mapcon_number_of_maps
    from phcpy.phcpy2c2 import py2c_mapcon_degree_of_map
    topdim = py2c_mapcon_top_dimension()
    print 'the top dimension :', topdim
    for dim in range(topdim, -1, -1):
        nbmaps = py2c_mapcon_number_of_maps(dim)
        print 'number of maps of dimension', dim, ':', nbmaps
        for ind in range(1, nbmaps+1):
            print 'monomial map', ind, 'of dimension', dim, ':'
            write_monomial_map(dim, ind, nbvar)
            degmap = py2c_mapcon_degree_of_map(dim, ind)
            print 'degree of map', ind, ':', degmap

def solve_binomials(nbvar, pols, silent=True, puretopdim=False):
    r"""
    If the system given in *pols* as a list of strings in as many
    variables as the value of *nbvar* is a binomial system
    (that is: it has exactly two monomials with a nonzero coefficient
    in every equation), then this function will return monomial maps
    to represent the solution sets.
    By default, *silent* is True and no additional output is written.
    If only the expected pure top dimensional solution sets are of interest,
    then switch the default *puretopdim* to True for faster results.
    The expected top dimension equals the number of variables minus
    the number of equations.
    """
    from phcpy.phcpy2c2 import py2c_syscon_write_standard_Laurent_system
    from phcpy.phcpy2c2 import py2c_mapcon_solve_system
    from phcpy.phcpy2c2 import py2c_mapcon_write_maps
    from phcpy.phcpy2c2 import py2c_mapcon_clear_maps
    store_laurent_system(nbvar, pols)
    if not silent:
        print 'the polynomials on input :'
        py2c_syscon_write_standard_Laurent_system()
    isbin = is_binomial_system(silent)
    result = []
    if isbin:
        py2c_mapcon_solve_system(puretopdim)
        if not silent:
            py2c_mapcon_write_maps()
            write_monomial_maps(nbvar)
        result = monomial_map_solutions(nbvar)
        py2c_mapcon_clear_maps()
    return result

def test():
    """
    Solves a binomial system which has the x-axis, the yz-plane,
    and the twisted cubic as solution components.
    The yz-plane is a solution set of the unexpected dimension 2.
    """
    twisted2 = ['x**2*y - z*x;', 'x**2*z - y**2*x;']
    maps = solve_binomials(3, twisted2, silent=False)
    for solmap in maps:
        print solmap
    # note that the expected top dimension is 1,
    # the output below contains one curve on a 2-dimensional set
    print 'looking only for expected pure dimensional sets ...'
    maps = solve_binomials(3, twisted2, puretopdim=True)
    for solmap in maps:
        print solmap

if __name__ == "__main__":
    test()



"""
To install phcpy, the file libPHCpack must be in the folder phcpy,
in the same location where the modules of phcpy are installed.
"""
from distutils.core import setup

setup(
    name = 'PHCpy' ,
    author = 'Jan Verschelde' ,
    author_email = 'janv@uic.edu' ,
    description = 'a package for Polynomial Homotopy Continuation' ,
    url = 'https://github.com/janverschelde/PHCpack' ,
    version = '1.1.4' ,
    packages = ['phcpy'] ,
    py_modules = ['phcpy/version', 'phcpy/dimension', \
        'phcpy/polynomials', 'phcpy/solutions', 'phcpy/volumes', \
        'phcpy/solver', 'phcpy/examples', 'phcpy/families', \
        'phcpy/scaling', 'phcpy/starters', 'phcpy/homotopies', \
        'phcpy/trackers', 'phcpy/tropisms', 'phcpy/sweepers', \
        'phcpy/series', 'phcpy/curves', \
        'phcpy/deflation', 'phcpy/schubert', 'phcpy/sets', \
        'phcpy/cascades', 'phcpy/diagonal', 'phcpy/factor', \
        'phcpy/decomposition' , 'phcpy/binomials' ],
    license = 'GNU GENERAL PUBLIC LICENSE version 3' ,
    package_data = {'phcpy':['libPHCpack.so', 'libPHCpack.dll', \
        'libPHCpack.dylib']} ,
    platforms = ['linux', 'windows', 'macosx'] ,
    long_description=open('README.md').read()
)



def phccheck(polyname):
   from subprocess import Popen
   from phc_config import phc, phc_check
   from os import system, path, waitpid
   
   filename = "%s.poly"% polyname
   result = "%s.err"% polyname
   if path.isfile(result):
      r = 'rm -f %s'% result  # XXX remove outfile?
      s = system(r)

   p = Popen([phc, "-g", filename, result])
   s = waitpid(p.pid, 0)

   f = open(result, 'r')

   success_mark = 'PHC parsed input successfully on'
   success_mark_l = len(success_mark)
   
   poly_err = 1
   for line in f:
      if line[:success_mark_l] == success_mark:
         poly_err = 0
         break;

   if not poly_err:
      f.seek(0)
      degree = f.readline()
      f.close()

      r = 'rm -f %s'% result  # XXX remove outfile?
      s = system(r)
      
      for i in xrange(len(degree)):
         if degree[i] < '0' or degree[i] > '9':
            if i != len(degree)-1:
               return 0
            else:
               return int(degree[:i])
   else:
      f.close()
      return 0
   

def PHC_Solve(Uid,data):
   from subprocess import PIPE, Popen
   from os import setsid
   from phc_config import phc_cmd
   
   with open(Uid+'.poly', 'w') as f:
      f.write(data)
   f.close()
   with open(Uid+'.sol', 'w') as f:
      f.write(data)
   f.close()
   cmd = phc_cmd + " %s.sol %s.phc > %s &" %(Uid,Uid,Uid)
   proc = Popen(cmd, stdout=PIPE, shell=True, preexec_fn=setsid)
   return proc.pid

def PHC_Kill(Pid, file_id):
    import signal, os
    if Pid != '':
       #if os.path.getsize(file_id + ".poly") == os.path.getsize(file_id + ".sol"):
        print "Kill Pid = %s"%Pid
        try:
           os.kill(int(Pid), signal.SIGKILL)
        except OSError as ex:
           print "Process is finished"
        r = 'rm -f %s.sol %s.phc %s.pid %s' %(file_id, file_id, file_id, file_id)
        s = os.system(r)
          
## check whether a polynomial has been solved
#def PHC_Status(name):
#   from os import path, system
#   # solved, solution is writing
#   if path.isfile(name+".sol") and path.getsize(name+".poly") != path.getsize(name+".sol"): 
#      # XXX solution file writting might not be complete
#      if path.isfile(name+".pid"): # remove pid file
#         r = 'rm -f %s.pid'% name
#         s = system(r)
#      return 1
#   # not solved
#   else:
#      return 0

def processes():
   import subprocess
   import shlex
   proc1 = subprocess.Popen(['ps', 'aux'],stdout=subprocess.PIPE)
   proc2 = subprocess.Popen(['grep', '[p]hc64'],stdin=proc1.stdout,
                            stdout=subprocess.PIPE,stderr=subprocess.PIPE)
   proc1.stdout.close() # Allow proc1 to receive a SIGPIPE if proc2 exits.
   return proc2.communicate()[0].split('\n')



def check_file_exist(file_name):
   from os import path
   if path.isfile(file_name):
      return 1
   else:
      from phc_config import phc_debug
      if phc_debug:
         print "<p><font color='red'>The file %s doesn't exist. Try it again later.</font></p>"%file_name
      return 0

def get_file_id(Uid, poly_id):
   return "../users/%s/%s"%(Uid, poly_id)

def get_file_pid(file_id):
   pid = 0
   with open('%s.pid'%file_id,'r') as f:
      pid = int(f.read())
   return pid
      

def scp_upload(filename, destination):
   from os import waitpid
   from subprocess import Popen
#   myfile = "phc_html.py"
#   destination = "yxc@131.193.178.177:Desktop/"
   p = Popen(["scp",  filename, destination])
   sts = waitpid(p.pid, 0)
   print "scp uploading done"

def rm_exts(file_id, exts):
   """
   Delete all files_id.ext files
   """
   from os import remove, path
   
   for ext in exts:
      file_name = "%s.%s"%(file_id, ext)
      if path.isfile(file_name):
         remove(file_name)
 
def rm_all_exts(file_id):
   from os import system
   """
   Delete all files_id.*
   """
   r = 'rm -f %s.*'%file_id
   s = system(r)

def rm_without_exts(file_id, exts):
   """
   Delete all files_id.* except .ext files
   """
   from os import remove, path, listdir
   
   l = len(file_id) - 1
   
   for i in xrange(l+1):
      if i == l:
         path ='.'
         file_name = file_id
         break;
      elif file_id[l-i] == '/':
         file_name = file_id[-i:]
         path = file_id[:l-i]
         break;
   
   l = len(file_name)
   for f in listdir(path):
      if f[:l] == file_name:
         if f[l+1:] not in exts:
            remove(path+'/'+f)

def poly_file_id(Uid, poly_id):
   return "../users/%s/%s"%(Uid, poly_id)


# This part need to read every file, needs to be improve
def Deg(poly_id):
   file = open(poly_id+".poly",'r')
   line = file.next()
   file.close()
   try:
      deg = int(line)
   except ValueError:
      deg = -1
   return deg

def check_int(s):
   text_sym = ['\n','\r', '\t']
   l = len(s)-1
   for i in xrange(l):
      if s[i] > '9' or s[i] < '0':
         return 0

   if s[l] > '9' or s[l] < '0':
      if s[l] not in text_sym:
         return 0

   return 1

def Deg_Sols(poly_id):
   file = open(poly_id+".sol",'r')
   line = file.readline().split(' ')
   if len(line) == 2 and check_int(line[0]) and check_int(line[1]):
      sols = int(line[0])
      deg = int(line[1])
   else:
      for line in file:
         if line == "THE SOLUTIONS :\n":
            line = file.next()
            break

      for i in range(0, len(line)):
         if line[i] == ' ': 
            sols = int(line[:i])
            deg = int(line[i+1:])
            break
   file.close()
   return sols, deg

def Sols(Uid, poly_id):
   file_id = poly_file_id(Uid,poly_id)
   file = open("%.phc"%file_id,'r')
   for line in file:
      if line[0] == 'T':
         line = file.next()
         break
   for i in range(0, len(line)):
      if line[i] == ' ': 
         sols = int(line[:i])
         deg = int(line[i+1:])
         break
   file.close()
   return sols

#This part need to read every file, needs to be improve
def Slvtime(Uid, poly_id):
   file_id = poly_file_id(Uid,poly_id)
   file = open("%.phc"%file_id,'r')
   file.seek(-1024, 2)
   last = file.readlines()[-1].decode()
   file.close()
   slvtime = last[25:]
   for i in range(0, len(slvtime)):
      if slvtime[i] == 's':
         slvtime = slvtime[:i+1]
         break
   return slvtime

#This part need to read every file, needs to be improve
def Solve_Time(poly_id):
   file = open(poly_id+".phc",'r')
   file.seek(-1024, 2)
   last = file.readlines()[-1].decode()
   file.close()
   solvetime = last[25:]
   for i in range(0, len(solvetime)):
      if solvetime[i] == 's':
         solvetime = solvetime[:i+1]
         break
   return solvetime

def solve_time(poly_id):
   """
   read time information for phc file
   """
   file = open(poly_id+".phc",'r')
   n_steps = 3
   solvetime = [0.0 for i in xrange(n_steps + 1)]

   time_line = ["TIMING INFORMATION for Root Counting\n",
                "TIMING INFORMATION for Construction of Start System\n",
                "TIMING INFORMATION for continuation\n",
                "TIMING INFORMATION for Solving the polynomial system\n"]
   time_mark = "The elapsed time in seconds was";


   time_record = -1
   for line in file:
      if time_record != -1:
         line = line[len(time_mark):].strip()
         solvetime[time_record] = float(line.split(" =")[0])
         time_record = -1
      elif line in time_line:
         time_record = time_line.index(line)

   if solvetime[n_steps] == 0.0:
      for i in xrange(n_steps):
         solvetime[n_steps] += solvetime[i]

   return solvetime[n_steps]


def phc_start(startfile):
   """
   Get the start system from the solution file
   """
   from os import remove, path
   
   
   if path.isfile('%s.start'% startfile):
      remove('%s.start'% startfile)

   Start_System_Line = ["START SYSTEM : \n", " START SYSTEM :\n"]
   
   Start_System_Line_Len = len(Start_System_Line[0])
   
   f = open('%s.phc'% startfile,'r')
   
   g = open('%s.start'% startfile,'w')
   
   print startfile

   f.readline()
   
   breaknextloop = 0
   for line in f:
      if breaknextloop == 1:
         for x in line:
            if x != " " and x != "\n":
               breaknextloop = 2
               break
         if breaknextloop == 2:
            break
      if len(line)>Start_System_Line_Len and line[-Start_System_Line_Len:] in Start_System_Line:
         breaknextloop = 1
   
   if breaknextloop != 2:
      g.close()
      remove('%s.start'% startfile)
      return

   g.write(line)
   
   Solution_Start_Line = ["THE START SOLUTIONS : \n", "START SOLUTIONS : \n"]
   Solution_End_Line = ["HOMOTOPY PARAMETERS :\n", "TIMING INFORMATION for continuation\n"]
   
   for line in f:
      if line in Solution_Start_Line :
         g.write("THE SOLUTIONS : \n")
      elif line in Solution_End_Line:
         break
      else:
         g.write(line)
    
   f.close()
   g.close()



"""
The module series exports functions to compute power series solutions with
Newton's method in double, double double, or quad double precision.
"""

def replace_symbol(pol, idx):
    """
    In the polynomial pol, 
    replaces the first symbol by the symbol at place idx.
    """
    from phcpy.phcpy2c2 import py2c_syscon_string_of_symbols
    sbl = py2c_syscon_string_of_symbols()
    var = sbl.split(' ')
    result = pol.replace(var[0], var[idx-1])
    return result

def substitute_symbol(pols, idx):
    """
    Given in pols is a list of polynomials,
    replaces the first symbol by the symbol at place idx.
    """
    if idx == 1:
        return pols
    else:
        result = []
        for pol in pols:
            result.append(replace_symbol(pol, idx))
        return result

def standard_newton_series(pols, sols, idx=1, maxdeg=4, nbr=4, verbose=True):
    r"""
    Computes series in standard double precision for the polynomials
    in *pols*, where the leading coefficients are the solutions in *sols*.
    On entry are the following five parameters:

    *pols*: a list of string representations of polynomials,

    *sols*: a list of solutions of the polynomials in *pols*,

    *idx*: index of the series parameter, by default equals 1,

    *maxdeg*: maximal degree of the series,

    *nbr*: number of steps with Newton's method,

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import store_standard_system, load_standard_system
    from phcpy.phcpy2c2 import py2c_standard_Newton_series as newton
    from phcpy.phcpy2c2 import py2c_syspool_standard_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_standard_container
    from phcpy.phcpy2c2 import py2c_syspool_standard_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    store_standard_system(pols, nbvar=nbsym)
    store_standard_solutions(nbsym, sols)
    fail = newton(idx, maxdeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the execution of Newton's method."
        else:
            print "Computed %d series solutions." % size
    result = []
    for k in range(1, size+1):
        py2c_syspool_copy_to_standard_container(k)
        sersol = load_standard_system()
        result.append(substitute_symbol(sersol, idx))
    py2c_syspool_standard_clear()
    return result

def dobldobl_newton_series(pols, sols, idx=1, maxdeg=4, nbr=4, verbose=True):
    r"""
    Computes series in double double precision for the polynomials
    in *pols*, where the leading coefficients are the solutions in *sols*.
    On entry are the following five parameters:

    *pols*: a list of string representations of polynomials,

    *sols*: a list of solutions of the polynomials in *pols*,

    *idx*: index of the series parameter, by default equals 1,

    *maxdeg*: maximal degree of the series,

    *nbr*: number of steps with Newton's method,

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import store_dobldobl_system, load_dobldobl_system
    from phcpy.phcpy2c2 import py2c_dobldobl_Newton_series as newton
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_dobldobl_container
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    store_dobldobl_system(pols, nbvar=nbsym)
    store_dobldobl_solutions(nbsym, sols)
    fail = newton(idx, maxdeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the execution of Newton's method."
        else:
            print "Computed %d series solutions." % size
    result = []
    for k in range(1, size+1):
        py2c_syspool_copy_to_dobldobl_container(k)
        sersol = load_dobldobl_system()
        result.append(substitute_symbol(sersol, idx))
    py2c_syspool_dobldobl_clear()
    return result

def quaddobl_newton_series(pols, sols, idx=1, maxdeg=4, nbr=4, verbose=True):
    r"""
    Computes series in quad double precision for the polynomials
    in *pols*, where the leading coefficients are the solutions in *sols*.
    On entry are the following five parameters:

    *pols*: a list of string representations of polynomials,

    *sols*: a list of solutions of the polynomials in *pols*,

    *idx*: index of the series parameter, by default equals 1,

    *maxdeg*: maximal degree of the series,

    *nbr*: number of steps with Newton's method,

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import store_quaddobl_system, load_quaddobl_system
    from phcpy.phcpy2c2 import py2c_quaddobl_Newton_series as newton
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_quaddobl_container
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    store_quaddobl_system(pols, nbvar=nbsym)
    store_quaddobl_solutions(nbsym, sols)
    fail = newton(idx, maxdeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the execution of Newton's method."
        else:
            print "Computed %d series solutions." % size
    result = []
    for k in range(1, size+1):
        py2c_syspool_copy_to_quaddobl_container(k)
        sersol = load_quaddobl_system()
        result.append(substitute_symbol(sersol, idx))
    py2c_syspool_quaddobl_clear()
    return result

def checkin_newton_power_series(nbsym, lser, idx):
    """
    Given in nbsym the number of symbols in the polynomial system,
    in lser the list of leading terms in the series and 
    in idx the index of the parameter, returns True
    if nbsym = len(lser) if idx == 0, or otherwise
    if nbsym = len(lser) + 1 if idx != 0.
    An error message is written and False is returned
    if the above conditions are not satisfied.
    """
    if idx == 0:
        okay = (nbsym == len(lser))
    else:
        okay = (nbsym == len(lser) + 1)
    if not okay:
        if idx == 0:
            dim = nbsym
        else:
            dim = nbsym - 1
        print 'Wrong length of list of leading terms, should be', \
            str(dim) + '.'
       
    return okay   

def standard_newton_power_series(pols, lser, idx=1, maxdeg=4, nbr=4, \
    checkin=True, verbose=True):
    r"""
    Computes series in standard double precision for the polynomials
    in *pols*, where the leading terms are given in the list *lser*.
    On entry are the following five parameters:

    *pols*: a list of string representations of polynomials,

    *lser*: a list of polynomials in the series parameter (e.g.: t),
    for use as start terms in Newton's method,

    *idx*: index of the series parameter, by default equals 1,

    *maxdeg*: maximal degree of the power series,

    *nbr*: number of steps with Newton's method,

    *checkin*: checks whether the number of symbols in pols matches
    the length of the list lser if idx == 0, or is one less than 
    the length of the list lser if idx != 0.  If the conditions are
    not satisfied, then an error message is printed and lser is returned.

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_standard_system, load_standard_system
    from phcpy.phcpy2c2 import py2c_standard_Newton_power_series as newton
    from phcpy.phcpy2c2 import py2c_syspool_standard_init
    from phcpy.phcpy2c2 import py2c_syspool_standard_create
    from phcpy.phcpy2c2 import py2c_syspool_standard_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_standard_container
    from phcpy.phcpy2c2 import py2c_syspool_standard_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    if checkin:
        if not checkin_newton_power_series(nbsym, lser, idx):
            return lser
    store_standard_system(lser, nbvar=1)
    py2c_syspool_standard_init(1);
    py2c_syspool_standard_create(1);
    store_standard_system(pols, nbvar=nbsym)
    fail = newton(idx, maxdeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the execution of Newton's method."
        else:
            print "Computed one series solution."
    py2c_syspool_copy_to_standard_container(1)
    result = load_standard_system()
    result = substitute_symbol(result, idx)
    py2c_syspool_standard_clear()
    return result

def dobldobl_newton_power_series(pols, lser, idx=1, maxdeg=4, nbr=4, \
    checkin=True, verbose=True):
    r"""
    Computes series in double double precision for the polynomials
    in *pols*, where the leading terms are given in the list *lser*.
    On entry are the following five parameters:

    *pols*: a list of string representations of polynomials,

    *lser*: a list of polynomials in the series parameter (e.g.: t),
    for use as start terms in Newton's method,

    *idx*: index of the series parameter, by default equals 1,

    *maxdeg*: maximal degree of the series,

    *nbr*: number of steps with Newton's method,

    *checkin*: checks whether the number of symbols in pols matches
    the length of the list lser if idx == 0, or is one less than 
    the length of the list lser if idx != 0.  If the conditions are
    not satisfied, then an error message is printed and lser is returned.

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_dobldobl_system, load_dobldobl_system
    from phcpy.phcpy2c2 import py2c_dobldobl_Newton_power_series as newton
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_init
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_create
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_dobldobl_container
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    if checkin:
        if not checkin_newton_power_series(nbsym, lser, idx):
            return lser
    store_dobldobl_system(lser, nbvar=1)
    py2c_syspool_dobldobl_init(1);
    py2c_syspool_dobldobl_create(1);
    store_dobldobl_system(pols, nbvar=nbsym)
    fail = newton(idx, maxdeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the execution of Newton's method."
        else:
            print "Computed one series solution."
    py2c_syspool_copy_to_dobldobl_container(1)
    result = load_dobldobl_system()
    result = substitute_symbol(result, idx)
    py2c_syspool_dobldobl_clear()
    return result

def quaddobl_newton_power_series(pols, lser, idx=1, maxdeg=4, nbr=4, \
    checkin=True, verbose=True):
    r"""
    Computes series in quad double precision for the polynomials
    in *pols*, where the leading terms are given in the list *lser*.
    On entry are the following five parameters:

    *pols*: a list of string representations of polynomials,

    *lser*: a list of polynomials in the series parameter (e.g.: t),
    for use as start terms in Newton's method,

    *idx*: index of the series parameter, by default equals 1,

    *maxdeg*: maximal degree of the series,
 
    *nbr*: number of steps with Newton's method,

    *checkin*: checks whether the number of symbols in pols matches
    the length of the list lser if idx == 0, or is one less than 
    the length of the list lser if idx != 0.  If the conditions are
    not satisfied, then an error message is printed and lser is returned.

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_quaddobl_system, load_quaddobl_system
    from phcpy.phcpy2c2 import py2c_quaddobl_Newton_power_series as newton
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_init
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_create
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_quaddobl_container
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    if checkin:
        if not checkin_newton_power_series(nbsym, lser, idx):
            return lser
    store_quaddobl_system(lser, nbvar=1)
    py2c_syspool_quaddobl_init(1);
    py2c_syspool_quaddobl_create(1);
    store_quaddobl_system(pols, nbvar=nbsym)
    fail = newton(idx, maxdeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the execution of Newton's method."
        else:
            print "Computed one series solution."
    py2c_syspool_copy_to_quaddobl_container(1)
    result = load_quaddobl_system()
    result = substitute_symbol(result, idx)
    py2c_syspool_quaddobl_clear()
    return result

def make_fractions(pols):
    """
    Given a list of string representations for the numerator and
    denominator polynomials in its even and odd numbered indices,
    returns a list of string representations for the fractions.
    """
    result = []
    (nbr, idx) = (len(pols)/2, 0)
    for k in range(nbr):
        (num, den) = (pols[idx], pols[idx+1])
        idx = idx + 2
        frac = '(' + num[:-1] + ')/(' + den[:-1] + ')'
        result.append(frac)
    return result

def rational_forms(pols):
    """
    Given a list of lists of string representations for the numerators
    and denominators, returns the proper rational representations for
    the Pade approximants.
    """
    result = []
    for pol in pols:
        result.append(make_fractions(pol))
    return result

def standard_pade_approximants(pols, sols, idx=1, numdeg=2, dendeg=2, \
    nbr=4, verbose=True):
    r"""
    Computes Pade approximants based on the series in standard double 
    precision for the polynomials in *pols*, where the leading 
    coefficients of the series are the solutions in *sols*.
    On entry are the following seven parameters:

    *pols*: a list of string representations of polynomials,

    *sols*: a list of solutions of the polynomials in *pols*,

    *idx*: index of the series parameter, by default equals 1,

    *numdeg*: the degree of the numerator,

    *dendeg*: the degree of the denominator,

    *nbr*: number of steps with Newton's method,

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import store_standard_system, load_standard_system
    from phcpy.phcpy2c2 \
        import py2c_standard_Pade_approximant as Pade_approximants
    from phcpy.phcpy2c2 import py2c_syspool_standard_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_standard_container
    from phcpy.phcpy2c2 import py2c_syspool_standard_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    store_standard_system(pols, nbvar=nbsym)
    store_standard_solutions(nbsym, sols)
    fail = Pade_approximants(idx, numdeg, dendeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the Pade constructor."
        else:
            print "Computed %d Pade approximants." % size
    result = []
    for k in range(1, size+1):
        py2c_syspool_copy_to_standard_container(k)
        sersol = load_standard_system()
        substsersol = substitute_symbol(sersol, idx)
        result.append(make_fractions(substsersol))
    py2c_syspool_standard_clear()
    return result

def dobldobl_pade_approximants(pols, sols, idx=1, numdeg=2, dendeg=2, \
    nbr=4, verbose=True):
    r"""
    Computes Pade approximants based on the series in double double 
    precision for the polynomials in *pols*, where the leading 
    coefficients of the series are the solutions in *sols*.
    On entry are the following seven parameters:

    *pols*: a list of string representations of polynomials,

    *sols*: a list of solutions of the polynomials in *pols*,

    *idx*: index of the series parameter, by default equals 1,

    *numdeg*: the degree of the numerator,

    *dendeg*: the degree of the denominator,

    *nbr*: number of steps with Newton's method,

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import store_dobldobl_system, load_dobldobl_system
    from phcpy.phcpy2c2 \
        import py2c_dobldobl_Pade_approximant as Pade_approximants
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_dobldobl_container
    from phcpy.phcpy2c2 import py2c_syspool_dobldobl_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    store_dobldobl_system(pols, nbvar=nbsym)
    store_dobldobl_solutions(nbsym, sols)
    fail = Pade_approximants(idx, numdeg, dendeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the Pade constructor."
        else:
            print "Computed %d Pade approximants." % size
    result = []
    for k in range(1, size+1):
        py2c_syspool_copy_to_dobldobl_container(k)
        sersol = load_dobldobl_system()
        substsersol = substitute_symbol(sersol, idx)
        result.append(make_fractions(substsersol))
    py2c_syspool_dobldobl_clear()
    return result

def quaddobl_pade_approximants(pols, sols, idx=1, numdeg=2, dendeg=2, \
    nbr=4, verbose=True):
    r"""
    Computes Pade approximants based on the series in quad double 
    precision for the polynomials in *pols*, where the leading 
    coefficients of the series are the solutions in *sols*.
    On entry are the following seven parameters:

    *pols*: a list of string representations of polynomials,

    *sols*: a list of solutions of the polynomials in *pols*,

    *idx*: index of the series parameter, by default equals 1,

    *numdeg*: the degree of the numerator,

    *dendeg*: the degree of the denominator,

    *nbr*: number of steps with Newton's method,

    *verbose*: whether to write intermediate output to screen or not.

    On return is a list of lists of strings.  Each lists of strings
    represents the series solution for the variables in the list *pols*.
    """
    from phcpy.solver import number_of_symbols
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import store_quaddobl_system, load_quaddobl_system
    from phcpy.phcpy2c2 \
        import py2c_quaddobl_Pade_approximant as Pade_approximants
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_size as poolsize
    from phcpy.phcpy2c2 import py2c_syspool_copy_to_quaddobl_container
    from phcpy.phcpy2c2 import py2c_syspool_quaddobl_clear
    nbsym = number_of_symbols(pols)
    if verbose:
        print "the polynomials :"
        for pol in pols:
            print pol
        print "Number of variables :", nbsym
    store_quaddobl_system(pols, nbvar=nbsym)
    store_quaddobl_solutions(nbsym, sols)
    fail = Pade_approximants(idx, numdeg, dendeg, nbr, int(verbose))
    size = (-1 if fail else poolsize())
    if verbose:
        if size == -1:
            print "An error occurred in the Pade constructor."
        else:
            print "Computed %d Pade approximants." % size
    result = []
    for k in range(1, size+1):
        py2c_syspool_copy_to_quaddobl_container(k)
        sersol = load_quaddobl_system()
        substsersol = substitute_symbol(sersol, idx)
        result.append(make_fractions(substsersol))
    py2c_syspool_quaddobl_clear()
    return result

def viviani(prc='d'):
    """
    Returns the system which stores the Viviani curve,
    with some solutions intersected with a plane,
    in double ('d'), double double ('dd'), or quad double('qd') precision.
    """
    from phcpy.solver import solve
    pols = ['(1-s)*y + s*(y-1);', \
            'x^2 + y^2 + z^2 - 4;' , \
            '(x-1)^2 + y^2 - 1;', \
            's;']
    sols = solve(pols, verbose=False, precision=prc)
    print "The solutions on the Viviani curve :"
    for sol in sols:
        print sol
    return (pols[:3], sols)

def viviani2(precision='d'):
    """
    Computes the power series expansion for the Viviani curve,
    from a natural paramter perspective.
    The default precision is double ('d').  Other precisions
    are double double ('dd') and quad double ('qd').
    """
    pols = [ '2*t^2 - x;', \
             'x^2 + y^2 + z^2 - 4;' , \
             '(x-1)^2 + y^2 - 1;']
    lser = [ '2*t^2;', '2*t;', '2;']
    if precision == 'd':
        nser = standard_newton_power_series(pols, lser, maxdeg=12, nbr=8)
    elif precision == 'dd':
        nser = dobldobl_newton_power_series(pols, lser, maxdeg=12, nbr=8)
    elif precision == 'qd':
        nser = quaddobl_newton_power_series(pols, lser, maxdeg=12, nbr=8)
    else:
        print 'invalid argument for the precision'
    print nser

def apollonius(precision='d'):
    """
    Test on computing the power series at a double solution
    for the problem of Apolonius.
    The parameter t is the fourth variable, whence we call
    Newton's method with idx equal to four.
    """
    pols = [ 'x1^2 + 3*x2^2 - r^2 - 2*r - 1;', \
             'x1^2 + 3*x2^2 - r^2 - 4*x1 - 2*r + 3;', \
       '3*t^2 + x1^2 - 6*t*x2 + 3*x2^2 - r^2 + 6*t - 2*x1 - 6*x2 + 2*r + 3;']
 
    lser1 = ['1;', '1 + 0.536*t;', '1 + 0.904*t;']
    lser2 = ['1;', '1 + 7.464*t;', '1 + 11.196*t;']
    if precision == 'd':
        nser1 = standard_newton_power_series(pols, lser1, idx=4, nbr=7)
        nser2 = standard_newton_power_series(pols, lser2, idx=4, nbr=7)
    elif precision == 'dd':
        nser1 = dobldobl_newton_power_series(pols, lser1, idx=4, nbr=7)
        nser2 = dobldobl_newton_power_series(pols, lser2, idx=4, nbr=7)
    elif precision == 'qd':
        nser1 = quaddobl_newton_power_series(pols, lser1, idx=4, nbr=7)
        nser2 = quaddobl_newton_power_series(pols, lser2, idx=4, nbr=7)
    else:
        print 'invalid argument for the precision'
    print nser1
    print nser2

def example4pade(prc='d'):
    """
    The function f(z) = ((1 + 1/2*z)/(1 + 2*z))^(1/2) is
    a solution x(s) of (1-s)*(x^2 - 1) + s*(3*x^2 - 3/2) = 0
    """
    pols = ['(x^2 - 1)*(1-s) + (3*x^2 - 3/2)*s;', 's;']
    from phcpy.solver import solve
    sols = solve(pols, verbose=False, precision=prc)
    for sol in sols:
        print sol
    if prc == 'd':
        sers = standard_newton_series(pols[:1], sols, idx=2)
    else:
        sers = dobldobl_newton_series(pols[:1], sols, idx=2)
    print 'the series solutions :'
    for ser in sers:
        print ser
    if prc == 'd':
        pade = standard_pade_approximants(pols[:1], sols, idx=2)
    elif prc == 'dd':
        pade = dobldobl_pade_approximants(pols[:1], sols, idx=2)
    elif prc == 'qd':
        pade = quaddobl_pade_approximants(pols[:1], sols, idx=2)
    else:
        print 'wrong value for the precision'
    print 'the Pade approximants :'
    for pad in pade:
        print pad

def test(precision='d'):
    """
    Tests the application of Newton's method to compute power
    series solutions of a polynomial system.
    """
    (pols, sols) = viviani(precision)
    if precision == 'd':
        sersols = standard_newton_series(pols, sols)
    elif precision == 'dd':
        sersols = dobldobl_newton_series(pols, sols)
    elif precision == 'qd':
        sersols = quaddobl_newton_series(pols, sols)
    for series in sersols:
        print series

if __name__ == "__main__":
    #test('d')
    #test('dd')
    #test('qd')
    #viviani2('qd')
    #apollonius()
    example4pade('dd')



"""
Script to track 35,940 solution paths defined by an artificial-parameter
homotopy between the cyclic 10-roots problem and a random coefficient
start system for the cyclic 10-roots problem, in double precision,
either with the path trackers that apply algorithmic differentiation,
or accelerated by the GPU.  This script was used to prepare the timings
in the paper "Polynomial Homotopy Continuation on GPUs" (ACM CCA)
related to a software demonstration at ISSAC 2015.
"""
GPU = 1 # use the GPU
DIR = '/home/jan/Problems/GPUdata/MultiPath' # location of systems
from phcpy.phcpy2c \
import py2c_read_standard_target_system_from_file as read_target
from phcpy.phcpy2c \
import py2c_read_standard_start_system_from_file as read_start
cyc10tarfile = DIR + '/cyclic10.target'
cyc10stafile = DIR + '/cyclic10.start'
fail = read_target(len(cyc10tarfile), cyc10tarfile)
from phcpy.interface import load_standard_system as loadsys
from phcpy.interface import load_standard_solutions as loadsols
cyc10 = loadsys()
print 'the cyclic 10-roots problem :'
for pol in cyc10:
    print pol
fail = read_start(len(cyc10stafile), cyc10stafile)
cyc10q = loadsys()
print 'a start system for the cyclic 10-roots problem :'
for pol in cyc10q:
    print pol
cyc10qsols = loadsols()
print 'number of start solutions :', len(cyc10qsols)
print 'the first solution :'
print cyc10qsols[0]
print 'calling the path tracker...'
if(GPU == 0):
    from phcpy.trackers import ade_double_track
    cyc10sols = ade_double_track(cyc10,cyc10q,cyc10qsols,verbose=0)
else:
    from phcpy.trackers import gpu_double_track
    cyc10sols = gpu_double_track(cyc10,cyc10q,cyc10qsols,verbose=0)
print 'number of solutions :', len(cyc10sols)
for sol in cyc10sols:
    print sol
